import dis
import inspect
import unittest

import os
from collections import defaultdict
from pathlib import Path
from typing import Dict, Any

from parameterized import parameterized_class

from checkov.common.bridgecrew.check_type import CheckType
from checkov.common.bridgecrew.severities import BcSeverities, Severities
from checkov.common.graph.db_connectors.networkx.networkx_db_connector import NetworkxConnector
from checkov.common.graph.db_connectors.rustworkx.rustworkx_db_connector import RustworkxConnector
from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.dockerfile.base_dockerfile_check import BaseDockerfileCheck
from checkov.dockerfile.runner import Runner, get_files_definitions
from checkov.dockerfile.registry import registry
from checkov.runner_filter import RunnerFilter

RESOURCES_DIR = Path(__file__).parent / "resources"

@parameterized_class([
   {"db_connector": NetworkxConnector},
   {"db_connector": RustworkxConnector}
])
class TestRunnerValid(unittest.TestCase):
    def setUp(self) -> None:
        self.orig_checks = registry.checks

    def test_registry_has_type(self):
        self.assertEqual(registry.report_type, CheckType.DOCKERFILE)

    def test_runner_empty_dockerfile(self):
        current_dir = os.path.dirname(os.path.realpath(__file__))
        valid_dir_path = current_dir + "/resources/empty_dockerfile"
        runner = Runner(db_connector=self.db_connector())
        report = runner.run(root_folder=valid_dir_path, external_checks_dir=None,
                            runner_filter=RunnerFilter(framework='all'))
        self.assertEqual(report.failed_checks, [])
        self.assertEqual(report.parsing_errors, [])
        self.assertEqual(report.passed_checks, [])
        self.assertEqual(report.skipped_checks, [])

    def test_runner_name_variations(self):
        current_dir = os.path.dirname(os.path.realpath(__file__))
        valid_dir_path = current_dir + "/resources/name_variations"
        runner = Runner(db_connector=self.db_connector())
        report = runner.run(root_folder=valid_dir_path, external_checks_dir=None,
                            runner_filter=RunnerFilter(framework='all'))
        self.assertEqual(len(report.resources), 2)
        self.assertEqual(len([file for file in report.resources if 'Dockerfile.prod' in file]), 1)
        self.assertEqual(len([file for file in report.resources if 'prod.dockerfile' in file]), 1)

    def test_runner_failing_check(self):
        current_dir = os.path.dirname(os.path.realpath(__file__))
        valid_dir_path = current_dir + "/resources/expose_port/fail"
        runner = Runner(db_connector=self.db_connector())
        report = runner.run(root_folder=valid_dir_path, external_checks_dir=None,
                            runner_filter=RunnerFilter(framework='all',checks=['CKV_DOCKER_1']))
        self.assertEqual(len(report.failed_checks), 1)
        self.assertEqual(report.parsing_errors, [])
        self.assertEqual(report.passed_checks, [])
        self.assertEqual(report.skipped_checks, [])

    def test_runner_honors_enforcement_rules(self):
        current_dir = os.path.dirname(os.path.realpath(__file__))
        valid_dir_path = current_dir + "/resources/expose_port/fail"
        runner = Runner(db_connector=self.db_connector())
        filter = RunnerFilter(framework=['dockerfile'], use_enforcement_rules=True)
        # this is not quite a true test, because the checks don't have severities. However, this shows that the check registry
        # passes the report type properly to RunnerFilter.should_run_check, and we have tests for that method
        filter.enforcement_rule_configs = {CheckType.DOCKERFILE: Severities[BcSeverities.OFF]}
        report = runner.run(root_folder=valid_dir_path, external_checks_dir=None,
                            runner_filter=filter)
        self.assertEqual(len(report.failed_checks), 0)
        self.assertEqual(len(report.parsing_errors), 0)
        self.assertEqual(len(report.passed_checks), 0)
        self.assertEqual(len(report.skipped_checks), 0)

    def test_runner_failing_check_with_file_path(self):
        current_dir = os.path.dirname(os.path.realpath(__file__))
        valid_file_path = current_dir + "/resources/expose_port/fail/Dockerfile"
        runner = Runner(db_connector=self.db_connector())
        report = runner.run(
            files=[valid_file_path],
            external_checks_dir=None,
            runner_filter=RunnerFilter(framework="all", checks=["CKV_DOCKER_1"]),
        )
        self.assertEqual(len(report.failed_checks), 1)
        self.assertEqual(report.parsing_errors, [])
        self.assertEqual(report.passed_checks, [])
        self.assertEqual(report.skipped_checks, [])

    def test_runner_passing_check(self):
        current_dir = os.path.dirname(os.path.realpath(__file__))
        valid_dir_path = current_dir + "/resources/expose_port/pass"
        runner = Runner(db_connector=self.db_connector())
        report = runner.run(root_folder=valid_dir_path, external_checks_dir=None,
                            runner_filter=RunnerFilter(framework=["all"],checks=["CKV_DOCKER_1", "CKV2_DOCKER_1"]))
        self.assertEqual(len(report.passed_checks), 2)
        self.assertEqual(report.parsing_errors, [])
        self.assertEqual(report.failed_checks, [])
        self.assertEqual(report.skipped_checks, [])

        #  also check the abs file paths
        record_python = next(check for check in report.passed_checks if check.check_id == "CKV_DOCKER_1")
        assert record_python.file_abs_path.endswith("tests/dockerfile/resources/expose_port/pass/Dockerfile")
        record_graph = next(check for check in report.passed_checks if check.check_id == "CKV2_DOCKER_1")
        assert record_graph.file_abs_path.endswith("tests/dockerfile/resources/expose_port/pass/Dockerfile")

    def test_runner_skip_check(self):
        #  given
        valid_dir_path = Path(__file__).parent / "resources/expose_port/skip"

        # when
        report = Runner(db_connector=self.db_connector()).run(
            root_folder=str(valid_dir_path),
            external_checks_dir=None,
            runner_filter=RunnerFilter(
                framework=["dockerfile"],
                checks=["CKV_DOCKER_1", "CKV_DOCKER_5", "CKV_DOCKER_9", "CKV2_DOCKER_7"],
            ),
        )

        # then
        summary = report.get_summary()

        self.assertEqual(summary["passed"], 1)
        self.assertEqual(summary["failed"], 0)
        self.assertEqual(summary["skipped"], 3)
        self.assertEqual(summary["parsing_errors"], 0)

        expected_skipped_cehcks = [record.check_id for record in report.skipped_checks]
        self.assertCountEqual(["CKV_DOCKER_1", "CKV_DOCKER_5", "CKV2_DOCKER_7"], expected_skipped_cehcks)

    def test_record_has_severity(self):
        custom_check_id = "MY_CUSTOM_CHECK"

        registry.checks = defaultdict(list)

        class AnyFailingCheck(BaseDockerfileCheck):
            def __init__(self, *_, **__) -> None:
                super().__init__(
                    "this should fail",
                    custom_check_id,
                    [CheckCategories.ENCRYPTION],
                    ["RUN"]
                )

            def scan_entity_conf(self, conf: Dict[str, Any], entity_type: str):
                return CheckResult.FAILED, conf[0]

        check = AnyFailingCheck()
        check.severity = Severities[BcSeverities.LOW]

        current_dir = os.path.dirname(os.path.realpath(__file__))
        valid_dir_path = current_dir + "/resources/expose_port/fail"
        runner = Runner(db_connector=self.db_connector())
        report = runner.run(root_folder=valid_dir_path, external_checks_dir=None,
                            runner_filter=RunnerFilter(framework='dockerfile', checks=[custom_check_id]))

        self.assertEqual(report.failed_checks[0].severity, Severities[BcSeverities.LOW])

    def test_runner_severity(self):
        custom_check_id = "MY_CUSTOM_CHECK"

        registry.checks = defaultdict(list)

        class AnyFailingCheck(BaseDockerfileCheck):
            def __init__(self, *_, **__) -> None:
                super().__init__(
                    "this should fail",
                    custom_check_id,
                    [CheckCategories.ENCRYPTION],
                    ["RUN"]
                )

            def scan_entity_conf(self, conf: Dict[str, Any], entity_type: str):
                return CheckResult.FAILED, conf[0]

        check = AnyFailingCheck()
        check.severity = Severities[BcSeverities.HIGH]

        current_dir = os.path.dirname(os.path.realpath(__file__))
        valid_dir_path = current_dir + "/resources/expose_port/fail"
        runner = Runner(db_connector=self.db_connector())
        report = runner.run(root_folder=valid_dir_path, external_checks_dir=None,
                            runner_filter=RunnerFilter(framework='dockerfile', checks=['MEDIUM']))

        all_checks = report.failed_checks + report.passed_checks
        self.assertTrue(any(c.check_id == custom_check_id for c in all_checks))

    def test_runner_severity_omit(self):
        custom_check_id = "MY_CUSTOM_CHECK"

        registry.checks = defaultdict(list)

        class AnyFailingCheck(BaseDockerfileCheck):
            def __init__(self, *_, **__) -> None:
                super().__init__(
                    "this should fail",
                    custom_check_id,
                    [CheckCategories.ENCRYPTION],
                    ["RUN"]
                )

            def scan_entity_conf(self, conf: Dict[str, Any], entity_type: str):
                return CheckResult.FAILED, conf[0]

        check = AnyFailingCheck()
        check.severity = Severities[BcSeverities.HIGH]

        current_dir = os.path.dirname(os.path.realpath(__file__))
        valid_dir_path = current_dir + "/resources/expose_port/fail"
        runner = Runner(db_connector=self.db_connector())
        report = runner.run(root_folder=valid_dir_path, external_checks_dir=None,
                            runner_filter=RunnerFilter(framework='dockerfile', checks=['CRITICAL']))

        all_checks = report.failed_checks + report.passed_checks
        self.assertFalse(any(c.check_id == custom_check_id for c in all_checks))

    def test_runner_skip_severity(self):
        custom_check_id = "MY_CUSTOM_CHECK"

        registry.checks = defaultdict(list)

        class AnyFailingCheck(BaseDockerfileCheck):
            def __init__(self, *_, **__) -> None:
                super().__init__(
                    "this should fail",
                    custom_check_id,
                    [CheckCategories.ENCRYPTION],
                    ["RUN"]
                )

            def scan_entity_conf(self, conf: Dict[str, Any], entity_type: str):
                return CheckResult.FAILED, conf[0]

        check = AnyFailingCheck()
        check.severity = Severities[BcSeverities.LOW]

        current_dir = os.path.dirname(os.path.realpath(__file__))
        valid_dir_path = current_dir + "/resources/expose_port/fail"
        runner = Runner(db_connector=self.db_connector())
        report = runner.run(root_folder=valid_dir_path, external_checks_dir=None,
                            runner_filter=RunnerFilter(framework='dockerfile', checks=['MEDIUM']))

        all_checks = report.failed_checks + report.passed_checks
        self.assertFalse(any(c.check_id == custom_check_id for c in all_checks))

    def test_runner_skip_severity_omit(self):
        custom_check_id = "MY_CUSTOM_CHECK"

        registry.checks = defaultdict(list)

        class AnyFailingCheck(BaseDockerfileCheck):
            def __init__(self, *_, **__) -> None:
                super().__init__(
                    "this should fail",
                    custom_check_id,
                    [CheckCategories.ENCRYPTION],
                    ["RUN"]
                )

            def scan_entity_conf(self, conf: Dict[str, Any], entity_type: str):
                return CheckResult.FAILED, conf[0]

        check = AnyFailingCheck()
        check.severity = Severities[BcSeverities.HIGH]

        current_dir = os.path.dirname(os.path.realpath(__file__))
        valid_dir_path = current_dir + "/resources/expose_port/fail"
        runner = Runner(db_connector=self.db_connector())
        report = runner.run(root_folder=valid_dir_path, external_checks_dir=None,
                            runner_filter=RunnerFilter(framework='dockerfile', skip_checks=['MEDIUM']))

        all_checks = report.failed_checks + report.passed_checks
        self.assertTrue(any(c.check_id == custom_check_id for c in all_checks))

    def test_skip_wildcard_check(self):
        current_dir = os.path.dirname(os.path.realpath(__file__))
        valid_dir_path = current_dir + "/resources/wildcard_skip"
        runner = Runner(db_connector=self.db_connector())
        report = runner.run(root_folder=valid_dir_path, external_checks_dir=None,
                            runner_filter=RunnerFilter(framework=['dockerfile']))
        self.assertEqual(len(report.skipped_checks), 1)
        self.assertGreaterEqual(len(report.passed_checks), 1)
        self.assertGreaterEqual(len(report.failed_checks), 2)

    def test_wrong_check_imports(self):
        wrong_imports = ["arm", "cloudformation", "helm", "kubernetes", "serverless", "terraform"]
        check_imports = []

        checks_path = Path(inspect.getfile(Runner)).parent.joinpath("checks")
        for file in checks_path.rglob("*.py"):
            with file.open() as f:
                instructions = dis.get_instructions(f.read())
                import_names = [instr.argval for instr in instructions if "IMPORT_NAME" == instr.opname]

                for import_name in import_names:
                    wrong_import = next((import_name for x in wrong_imports if x in import_name), None)
                    if wrong_import:
                        check_imports.append({file.name: wrong_import})

        assert len(check_imports) == 0, f"Wrong imports were added: {check_imports}"

    def test_get_files_definitions(self):
        current_dir = os.path.dirname(os.path.realpath(__file__))
        valid_dockerfile = current_dir + "/resources/name_variations/Dockerfile.prod"
        not_valid_dockerfile = current_dir + "/resources/not_dockerfile/dockerfile.png"
        results = get_files_definitions([valid_dockerfile, not_valid_dockerfile])
        assert len(results) == 2
        assert len(results[0]) == 1 and list(results[0].keys())[0] == valid_dockerfile
        assert len(results[1]) == 1 and list(results[1].keys())[0] == valid_dockerfile

    def test_runner_extra_resources(self):
        # given
        test_file = RESOURCES_DIR / "name_variations/Dockerfile.prod"

        # when
        report = Runner(db_connector=self.db_connector()).run(
            files=[str(test_file)],
            runner_filter=RunnerFilter(framework=['dockerfile'], checks=["CKV_DOCKER_4"])  # chose a check, which will find nothing
        )

        # then
        summary = report.get_summary()

        self.assertEqual(summary["passed"], 0)
        self.assertEqual(summary["failed"], 0)
        self.assertEqual(summary["skipped"], 0)
        self.assertEqual(summary["parsing_errors"], 0)
        self.assertEqual(summary["resource_count"], 1)

        self.assertEqual(len(report.extra_resources), 1)
        extra_resource = next(iter(report.extra_resources))
        self.assertEqual(extra_resource.file_abs_path, str(test_file))
        self.assertTrue(extra_resource.file_path.endswith("Dockerfile.prod"))

    def test_runner_multiline(self):
        current_dir = os.path.dirname(os.path.realpath(__file__))
        valid_dir_path = current_dir + "/resources/multiline_command"
        runner = Runner(db_connector=self.db_connector())
        report = runner.run(root_folder=valid_dir_path, external_checks_dir=None,
                            runner_filter=RunnerFilter(framework='dockerfile', checks=['CKV_DOCKER_9']))
        self.assertEqual(len(report.failed_checks), 1)
        self.assertEqual(report.parsing_errors, [])
        self.assertEqual(report.passed_checks, [])
        self.assertEqual(report.skipped_checks, [])


    def tearDown(self) -> None:
        registry.checks = self.orig_checks


if __name__ == '__main__':
    unittest.main()



import os
from unittest import TestCase

from checkov.common.graph.db_connectors.networkx.networkx_db_connector import NetworkxConnector
from checkov.common.graph.db_connectors.rustworkx.rustworkx_db_connector import RustworkxConnector
from checkov.terraform.graph_builder.local_graph import TerraformLocalGraph
from checkov.terraform.tf_parser import TFParser

TEST_DIRNAME = os.path.dirname(os.path.realpath(__file__))


class TestGraphConnector(TestCase):
    def test_creating_networkx_graph(self):
        resources_dir = os.path.realpath(os.path.join(TEST_DIRNAME, '../resources/encryption'))
        hcl_config_parser = TFParser()
        module, _ = hcl_config_parser.parse_hcl_module(resources_dir, 'AWS')
        local_graph = TerraformLocalGraph(module)
        local_graph._create_vertices()
        nxc = NetworkxConnector()
        nxc.save_graph(local_graph)

    def test_creating_rustworkx_graph(self):
        resources_dir = os.path.realpath(os.path.join(TEST_DIRNAME, '../resources/encryption'))
        hcl_config_parser = TFParser()
        module, _ = hcl_config_parser.parse_hcl_module(resources_dir, 'AWS')
        local_graph = TerraformLocalGraph(module)
        local_graph._create_vertices()
        igc = RustworkxConnector()
        igc.save_graph(local_graph)



import pytest

from checkov.common.bridgecrew.check_type import CheckType
from checkov.common.models.enums import CheckResult
from checkov.common.output.report import Report
from checkov.common.output.secrets_record import SecretsRecord


@pytest.fixture
def mock_bc_integration():
    from checkov.common.bridgecrew.platform_integration import bc_integration
    bc_integration.bc_api_key = "abcd1234-abcd-1234-abcd-1234abcd1234"
    bc_integration.skip_download = False
    return bc_integration


@pytest.fixture
def mock_metadata_integration():
    from checkov.common.bridgecrew.integration_features.features.policy_metadata_integration import integration

    check_metadata = integration.check_metadata
    integration.check_metadata = {
        "CKV_SECRET_2": {
            "id": "BC_GIT_2",
            "checkovId": "CKV_SECRET_2",
        },
        "CKV_SECRET_6": {
            "id": "BC_GIT_6",
            "checkovId": "CKV_SECRET_6",
        }
    }

    yield

    integration.check_metadata = check_metadata


@pytest.fixture
def secrets_report() -> Report:
    kwargs = {'validation_status': 'mock', 'check_id': 'mock', 'check_name': 'mock', 'code_block': 'mock', 'file_path': 'mock',
              'file_line_range': 'mock', 'evaluations': 'mock', 'check_class': 'mock', 'file_abs_path': 'mock'}
    record_1 = SecretsRecord(bc_check_id="VIOLATION_1", resource="RESOURCE_1", check_result={"result": CheckResult.FAILED}, **kwargs)
    record_2 = SecretsRecord(bc_check_id="VIOLATION_2", resource="RESOURCE_2", check_result={"result": CheckResult.FAILED}, **kwargs)
    record_3 = SecretsRecord(bc_check_id="VIOLATION_3", resource="RESOURCE_3", check_result={"result": CheckResult.FAILED}, **kwargs)
    record_4 = SecretsRecord(bc_check_id="VIOLATION_4", resource="RESOURCE_4", check_result={"result": CheckResult.FAILED}, **kwargs)

    record_5 = SecretsRecord(bc_check_id="VIOLATION_1", resource="RESOURCE_1", check_result={"result": CheckResult.PASSED}, **kwargs)

    report = Report(CheckType.SECRETS)
    report.add_record(record_1)
    report.add_record(record_2)
    report.add_record(record_3)
    report.add_record(record_4)
    report.add_record(record_5)

    return report


@pytest.fixture
def secrets_report_invalid_status() -> Report:
    kwargs = {'check_id': 'mock', 'check_name': 'mock', 'code_block': 'mock',
              'file_path': 'mock',
              'file_line_range': 'mock', 'evaluations': 'mock', 'check_class': 'mock', 'file_abs_path': 'mock'}
    record_1 = SecretsRecord(bc_check_id="VIOLATION_1", resource="RESOURCE_1",
                             check_result={"result": CheckResult.FAILED}, validation_status='Invalid', **kwargs)
    record_2 = SecretsRecord(bc_check_id="VIOLATION_2", resource="RESOURCE_2",
                             check_result={"result": CheckResult.FAILED}, validation_status='Invalid', **kwargs)
    record_3 = SecretsRecord(bc_check_id="VIOLATION_3", resource="RESOURCE_3",
                             check_result={"result": CheckResult.FAILED}, validation_status='Invalid', **kwargs)
    record_4 = SecretsRecord(bc_check_id="VIOLATION_4", resource="RESOURCE_4",
                             check_result={"result": CheckResult.FAILED}, validation_status='Invalid', **kwargs)

    record_5 = SecretsRecord(bc_check_id="VIOLATION_1", resource="RESOURCE_1",
                             check_result={"result": CheckResult.PASSED}, validation_status='Invalid', **kwargs)

    report = Report(CheckType.SECRETS)
    report.add_record(record_1)
    report.add_record(record_2)
    report.add_record(record_3)
    report.add_record(record_4)
    report.add_record(record_5)

    return report



from aws_cdk import core
from aws_cdk import aws_docdb as docdb

class MyDocDBParameterGroupStack(core.Stack):
    def __init__(self, scope: core.Construct, id: str, **kwargs) -> None:
        super().__init__(scope, id, **kwargs)

        # Define DocDB Cluster Parameter Group with 'tls' parameter set to 'disabled'
        docdb.CfnDBClusterParameterGroup(
            self, 'MyDocDBClusterParameterGroup',
            description='My DocDB Parameter Group',
            family='docdb4.0',
            parameters={
                'tls': 'enabled'
            }
            # Other properties as needed
        )

app = core.App()
MyDocDBParameterGroupStack(app, "MyDocDBParameterGroupStack")
app.synth()



import unittest

import hcl2

from checkov.terraform.checks.resource.azure.AKSUsesAzurePoliciesAddon import check
from checkov.common.models.enums import CheckResult


class TestAKSUsesAzurePoliciesAddon(unittest.TestCase):

    def test_failure(self):
        hcl_res = hcl2.loads("""
                resource "azurerm_kubernetes_cluster" "example" {
                  name                = "example-aks1"
                  location            = azurerm_resource_group.example.location
                  resource_group_name = azurerm_resource_group.example.name
                  dns_prefix          = "exampleaks1"
                  
                  default_node_pool {
                    name       = "default"
                    node_count = 1
                    vm_size    = "Standard_D2_v2"
                  }
                
                  identity {
                    type = "SystemAssigned"
                  }
                
                  tags = {
                    Environment = "Production"
                  }
                }
        """)
        resource_conf = hcl_res['resource'][0]['azurerm_kubernetes_cluster']['example']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_failure2(self):
        hcl_res = hcl2.loads("""
                resource "azurerm_kubernetes_cluster" "example" {
                  name                = "example-aks1"
                  location            = azurerm_resource_group.example.location
                  resource_group_name = azurerm_resource_group.example.name
                  dns_prefix          = "exampleaks1"

                  addon_profile {
                    azure_policy {
                      enabled = false
                    }
                  }

                  default_node_pool {
                    name       = "default"
                    node_count = 1
                    vm_size    = "Standard_D2_v2"
                  }

                  identity {
                    type = "SystemAssigned"
                  }

                  tags = {
                    Environment = "Production"
                  }
                }
        """)
        resource_conf = hcl_res['resource'][0]['azurerm_kubernetes_cluster']['example']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)


    def test_failure3(self):
        hcl_res = hcl2.loads("""
                resource "azurerm_kubernetes_cluster" "example" {
                  name                = "example-aks1"
                  location            = azurerm_resource_group.example.location
                  resource_group_name = azurerm_resource_group.example.name
                  dns_prefix          = "exampleaks1"

                  azure_policy {
                    enabled = true
                  }

                  default_node_pool {
                    name       = "default"
                    node_count = 1
                    vm_size    = "Standard_D2_v2"
                  }

                  identity {
                    type = "SystemAssigned"
                  }

                  tags = {
                    Environment = "Production"
                  }
                }
        """)
        resource_conf = hcl_res['resource'][0]['azurerm_kubernetes_cluster']['example']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)


    def test_failure4(self):
        hcl_res = hcl2.loads("""
                resource "azurerm_kubernetes_cluster" "example" {
                  name                = "example-aks1"
                  location            = azurerm_resource_group.example.location
                  resource_group_name = azurerm_resource_group.example.name
                  dns_prefix          = "exampleaks1"

                  azure_policy_enabled = false

                  default_node_pool {
                    name       = "default"
                    node_count = 1
                    vm_size    = "Standard_D2_v2"
                  }

                  identity {
                    type = "SystemAssigned"
                  }

                  tags = {
                    Environment = "Production"
                  }
                }
        """)
        resource_conf = hcl_res['resource'][0]['azurerm_kubernetes_cluster']['example']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)


    def test_success(self):
        hcl_res = hcl2.loads("""
                resource "azurerm_kubernetes_cluster" "example" {
                  name                = "example-aks1"
                  location            = azurerm_resource_group.example.location
                  resource_group_name = azurerm_resource_group.example.name
                  dns_prefix          = "exampleaks1"
                  
                  addon_profile {
                    azure_policy {
                      enabled = true
                    }
                  }

                  default_node_pool {
                    name       = "default"
                    node_count = 1
                    vm_size    = "Standard_D2_v2"
                  }

                  identity {
                    type = "SystemAssigned"
                  }

                  tags = {
                    Environment = "Production"
                  }
                }
        """)
        resource_conf = hcl_res['resource'][0]['azurerm_kubernetes_cluster']['example']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)


    def test_success2(self):
        hcl_res = hcl2.loads("""
                resource "azurerm_kubernetes_cluster" "example" {
                  name                = "example-aks1"
                  location            = azurerm_resource_group.example.location
                  resource_group_name = azurerm_resource_group.example.name
                  dns_prefix          = "exampleaks1"
                  
                  azure_policy_enabled = true

                  default_node_pool {
                    name       = "default"
                    node_count = 1
                    vm_size    = "Standard_D2_v2"
                  }
                
                  identity {
                    type = "SystemAssigned"
                  }
                
                  tags = {
                    Environment = "Production"
                  }
                }
        """)
        resource_conf = hcl_res['resource'][0]['azurerm_kubernetes_cluster']['example']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)


if __name__ == '__main__':
    unittest.main()



import unittest

import hcl2

from checkov.terraform.checks.resource.azure.KeyExpirationDate import check
from checkov.common.models.enums import CheckResult


class TestKeyExpirationDate(unittest.TestCase):

    def test_failure(self):
        hcl_res = hcl2.loads("""
            resource "azurerm_key_vault_key" "generated" {
              name         = "generated-certificate"
              key_vault_id = azurerm_key_vault.example.id
              key_type     = "RSA"
              key_size     = 2048
            
              key_opts = [
                "decrypt",
                "encrypt",
                "sign",
                "unwrapKey",
                "verify",
                "wrapKey",
              ]
            }
                """)
        resource_conf = hcl_res['resource'][0]['azurerm_key_vault_key']['generated']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_success(self):
        hcl_res = hcl2.loads("""
            resource "azurerm_key_vault_key" "generated" {
              name         = "generated-certificate"
              key_vault_id = azurerm_key_vault.example.id
              key_type     = "RSA"
              key_size     = 2048
            
              key_opts = [
                "decrypt",
                "encrypt",
                "sign",
                "unwrapKey",
                "verify",
                "wrapKey",
              ]
              expiration_date = "2020-12-30T20:00:00Z"
            }
                """)
        resource_conf = hcl_res['resource'][0]['azurerm_key_vault_key']['generated']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)


if __name__ == '__main__':
    unittest.main()



from aws_cdk import core
from aws_cdk import aws_redshift as redshift

class MyRedshiftClusterParameterGroupStack(core.Stack):
    def __init__(self, scope: core.Construct, id: str, **kwargs) -> None:
        super().__init__(scope, id, **kwargs)

        # Define Redshift Cluster Parameter Group with require_ssl parameter
        redshift.CfnClusterParameterGroup(
            self, 'MyRedshiftClusterParameterGroup',
            description='My Redshift Parameter Group',
            parameter_group_family='redshift-1.0',
            parameters=[
                redshift.CfnClusterParameterGroup.ParameterProperty(
                    parameter_name='require_ssl',
                    parameter_value='false'
                )
                # Add other parameters if needed
            ]
        )

app = core.App()
MyRedshiftClusterParameterGroupStack(app, "MyRedshiftClusterParameterGroupStack")
app.synth()

class MyRedshiftClusterParameterGroupStack2(core.Stack):
    def __init__(self, scope: core.Construct, id: str, **kwargs) -> None:
        super().__init__(scope, id, **kwargs)

        # Define Redshift Cluster Parameter Group with require_ssl parameter
        redshift.CfnClusterParameterGroup(
            self, 'MyRedshiftClusterParameterGroup',
            description='My Redshift Parameter Group',
            parameter_group_family='redshift-1.0',
            parameters=[
                redshift.CfnClusterParameterGroup.ParameterProperty(
                    parameter_name='abc',
                    parameter_value='true'
                )
                # Add other parameters if needed
            ]
        )

app = core.App()
MyRedshiftClusterParameterGroupStack2(app, "MyRedshiftClusterParameterGroupStack2")
app.synth()


from pathlib import Path
from packaging import version as packaging_version
from checkov.common.bridgecrew.severities import Severities, BcSeverities
from checkov.common.models.enums import CheckResult

EXAMPLES_DIR = Path(__file__).parent / "examples"


def test_run(sca_package_report_dt):
    # given
    report = next(sca_package_report_dt)
    # then
    assert report.check_type == "sca_package"

    assert report.resources == {'package-lock.json.helmet-csp', 'requirements.txt.grpcio_tools', 'requirements.txt.gevent', 'requirements.txt.pymysql', 'requirements.txt.pyyaml', 'requirements.txt.sanic', 'package-lock.json.nconf', 'requirements.txt.pyramid', 'requirements.txt.multidict', 'requirements.txt.aiohttp', 'package-lock.json.cypress', 'requirements.txt.testcontainers', 'package-files/java/maven/normal/pom.xml.junit_junit', 'requirements.txt.requests', 'requirements.txt.tornado', 'package-lock.json.uglify-js', 'package-files/yarn/package.json.node-esapi', 'package-files/yarn/package.json.marked', 'package-lock.json.decode-uri-component', 'package-lock.json.mongodb', 'requirements.txt.django', 'package-lock.json.debug', 'requirements.txt.pika', 'requirements.txt.yarl', 'requirements.txt.protobuf', 'requirements.txt.celery', 'requirements.txt.uvicorn', 'requirements.txt.mysqlclient', 'requirements.txt.elasticsearch', 'package-files/yarn/package.json.needle', 'requirements.txt.fastapi', 'package-lock.json.marked', 'requirements.txt.pymongo', 'package-lock.json.bson', 'requirements.txt.redis', 'package-lock.json.minimist', 'package-lock.json.mocha', 'requirements.txt.kafka-python', 'requirements.txt.wrapt', 'requirements.txt.hug', 'package-lock.json.unset-value', 'package-lock.json.glob-parent', 'package-lock.json.grunt', 'requirements.txt.flask', 'package-files/yarn/package.json.bcrypt-nodejs', 'requirements.txt.contextvars', 'requirements.txt.werkzeug', 'requirements.txt.packaging', 'package-files/java/maven/normal/pom.xml.org.apache.logging.log4j_log4j-core', 'requirements.txt.gunicorn', 'requirements.txt.urllib3', 'requirements.txt.psycopg', 'requirements.txt.streamlit', 'package-files/java/maven/normal/pom.xml.commons-collections_commons-collections'}
    assert len(report.passed_checks) == 34
    assert len(report.failed_checks) == 56
    assert len(report.skipped_checks) == 0
    assert len(report.parsing_errors) == 0

    cve_record = next((c for c in report.failed_checks if
                       c.resource == "package-lock.json.bson"
                       and c.vulnerability_details.get('root_package_name', "") == 'mongodb'
                       and c.check_name == "SCA package scan"), None)
    assert cve_record is not None
    assert cve_record.bc_check_id == 'BC_CVE_2019_2391'
    assert cve_record.check_id == 'CKV_CVE_2019_2391'
    assert cve_record.check_name == "SCA package scan"
    assert cve_record.check_result == {"result": CheckResult.FAILED}
    assert cve_record.code_block == [(0, 'bson: 1.0.9')]
    assert cve_record.description == (
        'Incorrect parsing of certain JSON input may result in js-bson not correctly serializing BSON. '
        'This may cause unexpected application behaviour including data disclosure. '
        'This issue affects: MongoDB Inc. js-bson library version 1.1.3 and prior to.'
    )
    assert cve_record.file_abs_path == f"{EXAMPLES_DIR}/package-lock.json"
    assert cve_record.file_line_range == [0, 0]
    assert cve_record.file_path == "/package-lock.json"
    assert cve_record.repo_file_path == "/tests/sca_package_2/examples/package-lock.json"
    assert cve_record.resource == 'package-lock.json.bson'
    assert cve_record.severity == Severities[BcSeverities.MEDIUM]
    assert cve_record.short_description == 'CVE-2019-2391 - bson: 1.0.9'
    assert cve_record.vulnerability_details["lowest_fixed_version"] == "1.1.4"
    assert cve_record.vulnerability_details["fixed_versions"] == [
        packaging_version.parse("1.1.4"),
    ]
    assert cve_record.vulnerability_details['root_package_name'] == 'mongodb'
    assert cve_record.vulnerability_details['root_package_version'] == '2.2.36'
    assert cve_record.vulnerability_details['root_package_fix_version'] == '3.1.13'



from unittest import mock

import pytest
from checkov.common.images.image_referencer import Image
from checkov.terraform.image_referencer.provider.azure import AzureTerraformProvider
from tests.graph_utils.utils import GRAPH_FRAMEWORKS, set_graph_by_graph_framework, \
    add_vertices_to_graph_by_graph_framework


@pytest.mark.parametrize("graph_framework", GRAPH_FRAMEWORKS)
def extract_images_from_resources(graph_framework):
    # given
    resource = {
            "file_path_": "/batch.bicep",
            "__end_line__": 26,
            "__start_line__": 1,
            "properties": {
                "virtualMachineConfiguration": {
                    "containerConfiguration": {
                        "containerImageNames": ["nginx", "python:3.9-alpine"],
                        "containerRegistries": {
                            "password": "myPassword",  # checkov:skip=CKV_SECRET_6 test secret
                            "registryServer": "myContainerRegistry.azurecr.io",
                            "username": "myUserName",
                        },
                        "type": "DockerCompatible",
                    },
                }
            },
            "resource_type": "Microsoft.Batch/batchAccounts/pools",
        }
    graph = set_graph_by_graph_framework(graph_framework)
    add_vertices_to_graph_by_graph_framework(graph_framework, resource, graph)

    # when
    azure_provider = AzureTerraformProvider(graph_connector=graph)
    images = azure_provider.extract_images_from_resources()

    # then
    assert images == [
        Image(file_path="/batch.bicep", name="nginx", start_line=1, end_line=26),
        Image(file_path="/batch.bicep", name="python:3.9-alpine", start_line=1, end_line=26),
    ]


@pytest.mark.parametrize('graph_framework', GRAPH_FRAMEWORKS)
def test_extract_images_from_resources_with_no_image(graph_framework):
    # given
    resource = {
        "file_path_": "/batch.bicep",
        "__end_line__": 26,
        "__start_line__": 1,
        "properties": {
            "virtualMachineConfiguration": {
                "containerConfiguration": {
                    "containerImageNames": [],
                    "containerRegistries": {
                        "password": "myPassword",
                        "registryServer": "myContainerRegistry.azurecr.io",
                        "username": "myUserName",
                    },
                    "type": "DockerCompatible",
                },
            }
        },
        "resource_type": "Microsoft.Batch/batchAccounts/pools",
    }
    graph = set_graph_by_graph_framework(graph_framework)
    add_vertices_to_graph_by_graph_framework(graph_framework, resource, graph)

    # when
    with mock.patch.dict('os.environ', {'CHECKOV_GRAPH_FRAMEWORK': graph_framework}):
        azure_provider = AzureTerraformProvider(graph_connector=graph)
        images = azure_provider.extract_images_from_resources()

    # then
    assert not images



import unittest

import hcl2

from checkov.common.models.enums import CheckResult
from checkov.terraform.checks.resource.openstack.SecurityGroupUnrestrictedIngress3389 import check


class TestSecurityGroupUnrestrictedIngress3389(unittest.TestCase):

    def test_failure_compute_secgroup_ipv4(self):
        hcl_res = hcl2.loads("""
        resource "openstack_compute_secgroup_v2" "secgroup_1" {
          name        = "my_secgroup"
          description = "my security group"

          rule {
            from_port   = 80
            to_port     = 80
            ip_protocol = "tcp"
            cidr        = "0.0.0.0/0"
          }

          rule {
            from_port   = 3389
            to_port     = 3389
            ip_protocol = "tcp"
            cidr        = "192.168.0.0/16"
          }

          rule {
            from_port   = 3389
            to_port     = 3389
            ip_protocol = "tcp"
            cidr        = "0.0.0.0/0"
          }
        }
        """)
        resource_conf = hcl_res['resource'][0]['openstack_compute_secgroup_v2']['secgroup_1']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_failure_compute_secgroup_port_range(self):
        hcl_res = hcl2.loads("""
        resource "openstack_compute_secgroup_v2" "secgroup_1" {
          name        = "my_secgroup"
          description = "my security group"
        
          rule {
            from_port   = 0
            to_port     = 65535
            ip_protocol = "tcp"
            cidr        = "0.0.0.0/0"
          }
        }
        """)
        resource_conf = hcl_res['resource'][0]['openstack_compute_secgroup_v2']['secgroup_1']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_failure_compute_secgroup_ipv6(self):
        hcl_res = hcl2.loads("""
        resource "openstack_compute_secgroup_v2" "secgroup_1" {
          name        = "my_secgroup"
          description = "my security group"
        
          rule {
            from_port   = 3389
            to_port     = 3389
            ip_protocol = "tcp"
            cidr        = "192.168.0.0/16"
          }
          rule {
            from_port   = 3389
            to_port     = 3389
            ip_protocol = "tcp"
            cidr        = "::/0"
          }
        }
        """)
        resource_conf = hcl_res['resource'][0]['openstack_compute_secgroup_v2']['secgroup_1']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_pass_compute_secgroup_different_port(self):
        hcl_res = hcl2.loads("""
        resource "openstack_compute_secgroup_v2" "secgroup_1" {
          name        = "my_secgroup"
          description = "my security group"

          rule {
            from_port   = 33890
            to_port     = 33890
            ip_protocol = "tcp"
            cidr        = "192.168.0.0/16"
          }

          rule {
            from_port   = 33890
            to_port     = 33890
            ip_protocol = "tcp"
            cidr        = "0.0.0.0/0"
          }
        }
        """)
        resource_conf = hcl_res['resource'][0]['openstack_compute_secgroup_v2']['secgroup_1']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)

    def test_pass_compute_secgroup_no_cidr(self):
        hcl_res = hcl2.loads("""
        resource "openstack_compute_secgroup_v2" "secgroup_1" {
          name        = "my_secgroup"
          description = "my security group"
        
          rule {
            from_port     = 3389
            to_port       = 3389
            ip_protocol   = "tcp"
            from_group_id = "5338c192-5118-11ec-bf63-0242ac130002"
          }
        }
        """)
        resource_conf = hcl_res['resource'][0]['openstack_compute_secgroup_v2']['secgroup_1']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)

    def test_pass_compute_secgroup_null_cidr(self):
        hcl_res = hcl2.loads("""
        resource "openstack_compute_secgroup_v2" "secgroup_1" {
          name        = "my_secgroup"
          description = "my security group"
        
          rule {
            from_port   = 3389
            to_port     = 3389
            ip_protocol = "tcp"
            cidr        = null
          }
        }
            """)
        resource_conf = hcl_res['resource'][0]['openstack_compute_secgroup_v2']['secgroup_1']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)

    def test_pass_compute_secgroup_cidr(self):
        hcl_res = hcl2.loads("""
        resource "openstack_compute_secgroup_v2" "secgroup_1" {
          name        = "my_secgroup"
          description = "my security group"
        
          rule {
            from_port   = 3389
            to_port     = 3389
            ip_protocol = "tcp"
            cidr        = "192.168.0.0/16"
          }
        }
        """)
        resource_conf = hcl_res['resource'][0]['openstack_compute_secgroup_v2']['secgroup_1']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)

    def test_pass_compute_secgroup_icmp(self):
        hcl_res = hcl2.loads("""
        resource "openstack_compute_secgroup_v2" "secgroup_1" {
          name        = "my_secgroup"
          description = "my security group"
        
          rule {
            from_port   = 3389
            to_port     = 3389
            ip_protocol = "icmp"
            cidr        = "0.0.0.0/0"
          }
        }
        """)

        resource_conf = hcl_res['resource'][0]['openstack_compute_secgroup_v2']['secgroup_1']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)


    def test_failure_networking_secgroup(self):
        hcl_res = hcl2.loads("""
        resource "openstack_networking_secgroup_v2" "secgroup_1" {
          name        = "secgroup_1"
          description = "My neutron security group"
        }
        
        resource "openstack_networking_secgroup_rule_v2" "ingress" {
          direction         = "ingress"
          ethertype         = "IPv4"
          protocol          = "tcp"
          port_range_min    = 3389
          port_range_max    = 3389
          remote_ip_prefix  = "0.0.0.0/0"
        }
        """)

        resource_conf = hcl_res['resource'][0]['openstack_networking_secgroup_v2']['secgroup_1']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)

        resource_conf = hcl_res['resource'][1]['openstack_networking_secgroup_rule_v2']['ingress']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_failure_networking_secgroup_port_range(self):
        hcl_res = hcl2.loads("""
        resource "openstack_networking_secgroup_rule_v2" "ingress" {
          direction         = "ingress"
          ethertype         = "IPv4"
          protocol          = "tcp"
          port_range_min    = 1
          port_range_max    = 65535
          remote_ip_prefix  = "0.0.0.0/0"
        }
        """)

        resource_conf = hcl_res['resource'][0]['openstack_networking_secgroup_rule_v2']['ingress']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_pass_networking_secgroup(self):
        hcl_res = hcl2.loads("""
        resource "openstack_networking_secgroup_rule_v2" "ingress" {
          direction         = "ingress"
          ethertype         = "IPv4"
          protocol          = "tcp"
          port_range_min    = 3389
          port_range_max    = 3389
          remote_ip_prefix  = "192.168.0.0/16"
          security_group_id = "${openstack_networking_secgroup_v2.secgroup_1.id}"
        }
        """)

        resource_conf = hcl_res['resource'][0]['openstack_networking_secgroup_rule_v2']['ingress']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)

    def test_pass_networking_secgroup_icmp(self):
        hcl_res = hcl2.loads("""
        resource "openstack_networking_secgroup_rule_v2" "ingress" {
          direction         = "ingress"
          ethertype         = "IPv4"
          protocol          = "icmp"
          port_range_min    = 3389
          port_range_max    = 3389
          remote_ip_prefix  = "0.0.0.0/0"
        }
        """)

        resource_conf = hcl_res['resource'][0]['openstack_networking_secgroup_rule_v2']['ingress']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)

    def test_unknown_networking_secgroup_egress(self):
        hcl_res = hcl2.loads("""
        resource "openstack_networking_secgroup_rule_v2" "egress" {
          direction         = "egress"
          ethertype         = "IPv4"
          protocol          = "tcp"
          port_range_min    = 3389
          port_range_max    = 3389
          remote_ip_prefix  = "0.0.0.0/0"
        }
        """)

        resource_conf = hcl_res['resource'][0]['openstack_networking_secgroup_rule_v2']['egress']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.UNKNOWN, scan_result)

    def test_pass_networking_secgroup_source_sg(self):
        hcl_res = hcl2.loads("""
        resource "openstack_networking_secgroup_rule_v2" "ingress" {
          direction         = "ingress"
          ethertype         = "IPv4"
          protocol          = "tcp"
          port_range_min    = 3389
          port_range_max    = 3389
          security_group_id = "${openstack_networking_secgroup_v2.secgroup_1.id}"
        }
        """)

        resource_conf = hcl_res['resource'][0]['openstack_networking_secgroup_rule_v2']['ingress']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)

    def test_pass_networking_secgroup_different_port(self):
        hcl_res = hcl2.loads("""
        resource "openstack_networking_secgroup_rule_v2" "ingress" {
          direction         = "ingress"
          ethertype         = "IPv4"
          protocol          = "tcp"
          port_range_min    = 33890
          port_range_max    = 33890
          remote_ip_prefix  = "0.0.0.0/0"
        }
        """)

        resource_conf = hcl_res['resource'][0]['openstack_networking_secgroup_rule_v2']['ingress']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)


if __name__ == '__main__':
    unittest.main()



from aws_cdk import core
from aws_cdk import aws_cloudtrail as cloudtrail
from aws_cdk import aws_iam as iam

class CloudTrailStack(core.Stack):

    def __init__(self, scope: core.Construct, id: str, **kwargs) -> None:
        super().__init__(scope, id, **kwargs)

        # Create an AWS CloudTrail trail using CfnTrail
        trail = cloudtrail.CfnTrail(
            self,
            "MyCloudTrail",
            is_logging=True,
            enable_log_file_validation=False,
            management_events=[
                cloudtrail.ReadWriteType.WRITE_ONLY,
            ],
            include_global_service_events=True,
        )

app = core.App()
CloudTrailStack(app, "CloudTrailStack")
app.synth()



from aws_cdk import core
from aws_cdk import aws_secretsmanager as secretsmanager

class MySecretsStack(core.Stack):
    def __init__(self, scope: core.Construct, id: str, **kwargs) -> None:
        super().__init__(scope, id, **kwargs)

        # Define a SecretsManager secret with KMS key ID containing "aws/"
        my_secret = secretsmanager.Secret(
            self, 'MySecret',
            secret_name='MySecretName',
            kms_key_id='arn:aws:kms:REGION:ACCOUNT_ID:key/aws/KMS_KEY_ID'
        )

class MySecretsStack2(core.Stack):
    def __init__(self, scope: core.Construct, id: str, **kwargs) -> None:
        super().__init__(scope, id, **kwargs)

        my_secret = secretsmanager.Secret(
            self, 'MySecret',
            secret_name='MySecretName',
        )

app = core.App()
MySecretsStack(app, "MySecretsStack")
app.synth()



import unittest
import hcl2

from checkov.terraform.checks.resource.aws.ELBUsesSSL import check
from checkov.common.models.enums import CheckResult


class TestELBUsesSSL(unittest.TestCase):

    def test_failure_elb_one_listener(self):
        hcl_res = hcl2.loads("""
          resource "aws_elb" "test" {
              name               = "foobar-terraform-elb"
              availability_zones = ["us-west-2a", "us-west-2b", "us-west-2c"]
            
              listener {
                instance_port     = 8000
                instance_protocol = "http"
                lb_port           = 80
                lb_protocol       = "http"
              }
            
              health_check {
                healthy_threshold   = 2
                unhealthy_threshold = 2
                timeout             = 3
                target              = "HTTP:8000/"
                interval            = 30
              }
            
              instances                   = [aws_instance.foo.id]
              cross_zone_load_balancing   = true
              idle_timeout                = 400
              connection_draining         = true
              connection_draining_timeout = 400
          }
        """)
        resource_conf = hcl_res['resource'][0]['aws_elb']['test']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_failure_elb_multi_listener(self):
        hcl_res = hcl2.loads("""
          resource "aws_elb" "test" {
              name               = "foobar-terraform-elb"
              availability_zones = ["us-west-2a", "us-west-2b", "us-west-2c"]

              listener {
                instance_port     = 8000
                instance_protocol = "http"
                lb_port           = 80
                lb_protocol       = "http"
              }
              listener {
                instance_port      = 8000
                instance_protocol  = "http"
                lb_port            = 443
                lb_protocol        = "https"
                ssl_certificate_id = "arn:aws:iam::123456789012:server-certificate/certName"
              }    

              health_check {
                healthy_threshold   = 2
                unhealthy_threshold = 2
                timeout             = 3
                target              = "HTTP:8000/"
                interval            = 30
              }

              instances                   = [aws_instance.foo.id]
              cross_zone_load_balancing   = true
              idle_timeout                = 400
              connection_draining         = true
              connection_draining_timeout = 400
          }
        """)
        resource_conf = hcl_res['resource'][0]['aws_elb']['test']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_success_elb(self):
        hcl_res = hcl2.loads("""
        resource "aws_elb" "test" {
          name               = "foobar-terraform-elb"
          availability_zones = ["us-west-2a", "us-west-2b", "us-west-2c"]
          
          listener {
            instance_port      = 8000
            instance_protocol  = "http"
            lb_port            = 443
            lb_protocol        = "https"
            ssl_certificate_id = "arn:aws:iam::123456789012:server-certificate/certName"
          }           
        
          health_check {
            healthy_threshold   = 2
            unhealthy_threshold = 2
            timeout             = 3
            target              = "HTTP:8000/"
            interval            = 30
          }
        
          instances                   = [aws_instance.foo.id]
          cross_zone_load_balancing   = true
          idle_timeout                = 400
          connection_draining         = true
          connection_draining_timeout = 400
        }
        """)
        resource_conf = hcl_res['resource'][0]['aws_elb']['test']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)


if __name__ == '__main__':
    unittest.main()



from pathlib import Path


def test_graph_policies_packaging_requirements() -> None:
    """Checks, if all graph_checks folder have a __init__.py file

    It is needed to add a __init__.py in the base of a folder
    to treat it as a module and indicate Python to add it during
    installation of the package.
    """

    root_dir = Path(__file__).parents[4]

    for graph_dir in root_dir.rglob("graph_checks"):
        if ".mypy_cache" in graph_dir.parts:
            # skip paths, which are related to mypy
            continue

        assert (graph_dir / "__init__.py").exists()



import os
import unittest

from checkov.common.checks_infra.checks_parser import GraphCheckParser
from checkov.common.checks_infra.registry import Registry
from checkov.runner_filter import RunnerFilter
from pathlib import Path
from checkov.terraform.runner import Runner


class TestGraphChecks(unittest.TestCase):
    def test_internal_graph_checks_load(self):
        registry = Registry(parser=GraphCheckParser(), checks_dir=str(
            Path(__file__).parent.parent.parent.parent / "checkov" / "terraform" / "checks" / "graph_checks"))
        registry.load_checks()
        runner_filter = RunnerFilter()
        for check in registry.checks:
            self.assertFalse(runner_filter.is_external_check(check))
            # The BC ID should not be populated with a CKV2 ID
            self.assertIsNone(check.bc_id)

    def test_external_graph_check_load(self):
        runner = Runner()
        current_dir = os.path.dirname(os.path.realpath(__file__))
        runner.graph_registry.checks = []
        extra_checks_dir_path = [current_dir + "/extra_yaml_checks"]
        runner.load_external_checks(extra_checks_dir_path)
        self.assertEqual(len(runner.graph_registry.checks), 1)
        runner_filter = RunnerFilter()
        for check in runner.graph_registry.checks:
            self.assertTrue(runner_filter.is_external_check(check.id))
        runner.graph_registry.checks[:] = [check for check in runner.graph_registry.checks if "CUSTOM_GRAPH_AWS_1" not
                                           in check.id]

    def test_external_checks_and_graph_checks_load(self):
        runner = Runner()
        current_dir = os.path.dirname(os.path.realpath(__file__))
        runner_filter = RunnerFilter(framework=['terraform'])
        external_graph_checks = 0

        # with external yaml checks external graph registry checks count should be equal to the external graph checks
        extra_checks_dir_path = [current_dir + "/extra_checks", current_dir + "/extra_yaml_checks"]
        runner.run(root_folder=current_dir, external_checks_dir=extra_checks_dir_path,
                   runner_filter=runner_filter)
        for check in runner.graph_registry.checks:
            if runner_filter.is_external_check(check.id):
                external_graph_checks += 1
        self.assertGreater(len(runner.graph_registry.checks), 1)
        self.assertGreaterEqual(external_graph_checks, 1)
        runner.graph_registry.checks[:] = [check for check in runner.graph_registry.checks if
                                           "CUSTOM_GRAPH_AWS_1" not in check.id]


if __name__ == '__main__':
    unittest.main()



from typing import Dict, Any

import pytest

from checkov.common.bridgecrew.bc_source import SourceType
from checkov.common.bridgecrew.platform_integration import BcPlatformIntegration


@pytest.fixture()
def mock_bc_integration() -> BcPlatformIntegration:
    bc_integration = BcPlatformIntegration()
    bc_integration.bc_api_key = "abcd1234-abcd-1234-abcd-1234abcd1234"
    bc_integration.bc_api_url = "https://www.bridgecrew.cloud"
    bc_integration.setup_bridgecrew_credentials(
        repo_id="bridgecrewio/checkov",
        skip_fixes=True,
        skip_download=True,
        source=SourceType("Github", False),
        source_version="1.0",
        repo_branch="master",
        bc_api_url="https://www.bridgecrew.cloud"
    )
    return bc_integration


@pytest.fixture()
def package_scan_result() -> Dict[str, Any]:
    return {
        "repository": "/abs_path/to/app/requirements.txt",
        "passed": True,
        "packages": [
            {"type": "python", "name": "django", "version": "1.2", "path": "/abs_path/to/app/requirements.txt"}
        ],
        "complianceIssues": None,
        "complianceDistribution": {"critical": 0, "high": 0, "medium": 0, "low": 0, "total": 0},
        "vulnerabilities": [
            {
                "id": "CVE-2019-19844",
                "status": "fixed in 3.0.1, 2.2.9, 1.11.27",
                "cvss": 9.8,
                "vector": "CVSS:3.1/AV:N/AC:L/PR:N/UI:N/S:U/C:H/I:H/A:H",
                "description": "Django before 1.11.27, 2.x before 2.2.9, and 3.x before 3.0.1 allows account takeover.",
                "severity": "critical",
                "packageName": "django",
                "packageVersion": "1.2",
                "link": "https://nvd.nist.gov/vuln/detail/CVE-2019-19844",
                "riskFactors": ["Critical severity", "Has fix", "Attack complexity: low", "Attack vector: network"],
                "impactedVersions": ["\u003c1.11.27"],
                "publishedDate": "2019-12-18T20:15:00+01:00",
                "discoveredDate": "2019-12-18T19:15:00Z",
                "fixDate": "2019-12-18T20:15:00+01:00",
            }
        ],
        "vulnerabilityDistribution": {"critical": 1, "high": 0, "medium": 0, "low": 0, "total": 0},
    }


@pytest.fixture()
def docker_image_scan_result() -> Dict[str, Any]:
    return {
        "results": [
            {
                "id": "sha256:e70f7611f4d093d5f73026d23f7ab612aa1794abfb210ef1c549b225380d053b",
                "distro": "Debian GNU/Linux 11 (bullseye)",
                "distroRelease": "bullseye",
                "collections": ["All"],
                "packages": [
                    {"type": "os", "name": "mawk", "version": "1.3.4.20200120-2", "licenses": ["GPL-2"]},
                    {"type": "os", "name": "gzip", "version": "1.10-4", "licenses": ["GPL-3+"]},
                ],
                "compliances": [
                    {
                        "id": 41,
                        "title": "(CIS_Docker_v1.2.0 - 4.1) Image should be created with a non-root user",
                        "severity": "high",
                        "description": "It is a good practice to run the container as a non-root user, if possible. "
                        "Though user\nnamespace mapping is now available, if a user is already defined in "
                        "the container image, the\ncontainer is run as that user by default and specific"
                        " user namespace remapping is not\nrequired",
                    }
                ],
                "complianceDistribution": {"critical": 0, "high": 1, "medium": 0, "low": 0, "total": 1},
                "complianceScanPassed": True,
                "vulnerabilities": [
                    {
                        "id": "CVE-2022-23990",
                        "status": "fixed in 2.2.10-2+deb11u1",
                        "cvss": 9.8,
                        "vector": "CVSS:3.1/AV:N/AC:L/PR:N/UI:N/S:U/C:H/I:H/A:H",
                        "description": "Expat (aka libexpat) before 2.4.4 has an integer overflow in the doProlog "
                        "function.",
                        "severity": "critical",
                        "packageName": "expat",
                        "packageVersion": "2.2.10-2",
                        "link": "https://security-tracker.debian.org/tracker/CVE-2022-23990",
                        "riskFactors": [
                            "Attack complexity: low",
                            "Attack vector: network",
                            "Critical severity",
                            "Has fix",
                            "Recent vulnerability",
                        ],
                        "impactedVersions": ["<2.2.10-2+deb11u1"],
                        "publishedDate": "2022-01-26T21:15:00+02:00",
                        "discoveredDate": "2022-02-13T11:57:08+02:00",
                        "fixDate": "2022-01-26T21:15:00+02:00",
                    },
                    {
                        "id": "CVE-2022-23852",
                        "status": "fixed in 2.2.10-2+deb11u1",
                        "cvss": 9.8,
                        "vector": "CVSS:3.1/AV:N/AC:L/PR:N/UI:N/S:U/C:H/I:H/A:H",
                        "description": "Expat (aka libexpat) before 2.4.4 has a signed integer overflow in XML_GetBuffer, "
                        "for configurations with a nonzero XML_CONTEXT_BYTES.",
                        "severity": "critical",
                        "packageName": "expat",
                        "packageVersion": "2.2.10-2",
                        "link": "https://security-tracker.debian.org/tracker/CVE-2022-23852",
                        "riskFactors": [
                            "Attack complexity: low",
                            "Attack vector: network",
                            "Critical severity",
                            "Has fix",
                            "Recent vulnerability",
                        ],
                        "impactedVersions": ["<2.2.10-2+deb11u1"],
                        "publishedDate": "2022-01-24T04:15:00+02:00",
                        "discoveredDate": "2022-02-13T11:57:08+02:00",
                        "fixDate": "2022-01-24T04:15:00+02:00",
                    },
                ],
                "vulnerabilityDistribution": {"critical": 5, "high": 5, "medium": 0, "low": 14, "total": 24},
                "vulnerabilityScanPassed": True,
            }
        ]
    }



from aws_cdk import core
from aws_cdk import aws_rds as rds

class MyDBInstanceStack(core.Stack):
    def __init__(self, scope: core.Construct, id: str, **kwargs) -> None:
        super().__init__(scope, id, **kwargs)

        # Define RDS DB instance
        my_db_instance = rds.CfnDBInstance(
            self, 'MyDBInstance',
            engine='mysql',  # Change this to your desired engine type
            db_instance_class='db.t2.micro',
            allocated_storage=20,
            multi_az=True,
            # Other properties for your DB instance
        )

app = core.App()
MyDBInstanceStack(app, "MyDBInstanceStack")
app.synth()



import unittest

from checkov.terraform.checks.resource.aws.ECRImageScanning import check
from checkov.common.models.enums import CheckResult


class TestECRImageScanning(unittest.TestCase):

    def test_failure(self):
        resource_conf = {'name': ['bar'], 'image_tag_mutability': ['MUTABLE']}

        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_success(self):
        resource_conf = {'name': ['bar'], 'image_tag_mutability': ['MUTABLE'],
                         'image_scanning_configuration': [{'scan_on_push': [True]}]}
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)


if __name__ == '__main__':
    unittest.main()



import os
from unittest import mock
from unittest.case import TestCase

from checkov.cloudformation.graph_builder.graph_components.block_types import BlockType
from checkov.cloudformation.graph_manager import CloudformationGraphManager
from checkov.common.graph.db_connectors.networkx.networkx_db_connector import NetworkxConnector
from checkov.common.util.json_utils import get_jsonpath_from_evaluated_key

TEST_DIRNAME = os.path.dirname(os.path.realpath(__file__))


@mock.patch.dict(os.environ, {"RENDER_ASYNC_MAX_WORKERS": "50", "RENDER_VARIABLES_ASYNC": "False"})
class TestRenderer(TestCase):
    def test_render_ref(self):
        relative_path = './resources/variable_rendering/render_ref/'
        yaml_test_dir = os.path.realpath(os.path.join(TEST_DIRNAME, relative_path, 'yaml'))
        json_test_dir = os.path.realpath(os.path.join(TEST_DIRNAME, relative_path, 'json'))
        self.validate_render_ref(yaml_test_dir, 'yaml')
        self.validate_render_ref(json_test_dir, 'json')

    def validate_render_ref(self, test_dir: str, file_ext: str):
        graph_manager = CloudformationGraphManager(db_connector=NetworkxConnector())
        local_graph, _ = graph_manager.build_graph_from_source_directory(test_dir, render_variables=True)

        db_name_default_value = "db1"

        kms_master_key_id_expected_attributes = {'Default': None}
        db_name_expected_attributes = {'Default': db_name_default_value}
        my_source_queue_expected_attributes = {'KmsMasterKeyId.Ref': 'KmsMasterKeyId'}
        my_db_expected_attributes = {'DBName': db_name_default_value}
        my_db_instance_name_expected_attributes = {'Value.Ref': 'MyDB'}

        self.compare_vertex_attributes(local_graph, kms_master_key_id_expected_attributes, BlockType.PARAMETERS, 'KmsMasterKeyId')
        self.compare_vertex_attributes(local_graph, db_name_expected_attributes, BlockType.PARAMETERS, 'DBName')
        self.compare_vertex_attributes(local_graph, my_source_queue_expected_attributes, BlockType.RESOURCE, 'AWS::SQS::Queue.MySourceQueue')
        self.compare_vertex_attributes(local_graph, my_db_expected_attributes, BlockType.RESOURCE, 'AWS::RDS::DBInstance.MyDB')
        self.compare_vertex_attributes(local_graph, my_db_instance_name_expected_attributes, BlockType.OUTPUTS, 'MyDBInstanceName')

        kms_master_key_id_expected_breadcrumbs = {}
        db_name_expected_breadcrumbs = {}
        my_source_queue_expected_breadcrumbs = {}
        my_db_expected_breadcrumbs = {'DBName': [{'type': BlockType.PARAMETERS, 'name': 'DBName', 'path': os.path.join(test_dir, f'test.{file_ext}'), 'attribute_key': 'Default'}]}
        my_db_instance_name_expected_breadcrumbs = {}

        self.compare_vertex_breadcrumbs(local_graph, kms_master_key_id_expected_breadcrumbs, BlockType.PARAMETERS, 'KmsMasterKeyId')
        self.compare_vertex_breadcrumbs(local_graph, db_name_expected_breadcrumbs, BlockType.PARAMETERS, 'DBName')
        self.compare_vertex_breadcrumbs(local_graph, my_source_queue_expected_breadcrumbs, BlockType.RESOURCE, 'AWS::SQS::Queue.MySourceQueue')
        self.compare_vertex_breadcrumbs(local_graph, my_db_expected_breadcrumbs, BlockType.RESOURCE, 'AWS::RDS::DBInstance.MyDB')
        self.compare_vertex_breadcrumbs(local_graph, my_db_instance_name_expected_breadcrumbs, BlockType.OUTPUTS, 'MyDBInstanceName')


    def test_render_findinmap(self):
        relative_path = './resources/variable_rendering/render_findinmap/'
        yaml_test_dir = os.path.realpath(os.path.join(TEST_DIRNAME, relative_path, 'yaml'))
        json_test_dir = os.path.realpath(os.path.join(TEST_DIRNAME, relative_path, 'json'))
        self.validate_render_findinmap(yaml_test_dir, 'yaml')
        self.validate_render_findinmap(json_test_dir, 'json')

    def validate_render_findinmap(self, test_dir: str, file_ext: str):
        graph_manager = CloudformationGraphManager(db_connector=NetworkxConnector())
        local_graph, _ = graph_manager.build_graph_from_source_directory(test_dir, render_variables=True)

        region_map_expected_ami_value = "ami-0ff8a91507f77f867"

        region_map_expected_attributes = {'us-east-1.AMI': region_map_expected_ami_value}
        ec2instance_expected_attributes = {'ImageId': region_map_expected_ami_value}

        self.compare_vertex_attributes(local_graph, region_map_expected_attributes, BlockType.MAPPINGS, 'RegionMap')
        self.compare_vertex_attributes(local_graph, ec2instance_expected_attributes, BlockType.RESOURCE, 'AWS::EC2::Instance.EC2Instance')

        region_map_expected_breadcrumbs = {}
        ec2instance_expected_breadcrumbs = {'ImageId': [{'type': BlockType.MAPPINGS, 'name': 'RegionMap', 'path': os.path.join(test_dir, f'test.{file_ext}'), 'attribute_key': 'us-east-1.AMI'}]}

        self.compare_vertex_breadcrumbs(local_graph, region_map_expected_breadcrumbs, BlockType.MAPPINGS, 'RegionMap')
        self.compare_vertex_breadcrumbs(local_graph, ec2instance_expected_breadcrumbs, BlockType.RESOURCE, 'AWS::EC2::Instance.EC2Instance')


    def test_render_getatt(self):
        relative_path = './resources/variable_rendering/render_getatt/'
        yaml_test_dir = os.path.realpath(os.path.join(TEST_DIRNAME, relative_path, 'yaml'))
        json_test_dir = os.path.realpath(os.path.join(TEST_DIRNAME, relative_path, 'json'))
        self.validate_render_getatt(yaml_test_dir, 'yaml')
        self.validate_render_getatt(json_test_dir, 'json')

    def validate_render_getatt(self, test_dir: str, file_ext: str):
        graph_manager = CloudformationGraphManager(db_connector=NetworkxConnector())
        local_graph, _ = graph_manager.build_graph_from_source_directory(test_dir, render_variables=True)

        web_vpc_expected_cidr_block = "172.16.0.0/16"

        web_vpc_expected_attributes = {'CidrBlock': web_vpc_expected_cidr_block}
        my_sg_expected_attributes = {'SecurityGroupIngress.CidrIp': web_vpc_expected_cidr_block}
        web_vpc_default_sg_expected_attributes = {'Value.Fn::GetAtt': ['WebVPC', 'DefaultSecurityGroup']}

        self.compare_vertex_attributes(local_graph, web_vpc_expected_attributes, BlockType.RESOURCE, 'AWS::EC2::VPC.WebVPC')
        self.compare_vertex_attributes(local_graph, my_sg_expected_attributes, BlockType.RESOURCE, 'AWS::EC2::SecurityGroup.MySG')
        self.compare_vertex_attributes(local_graph, web_vpc_default_sg_expected_attributes, BlockType.OUTPUTS, 'WebVPCDefaultSg')

        web_vpc_expected_breadcrumbs = {}
        my_sg_expected_breadcrumbs = {
            "SecurityGroupIngress.0.CidrIp": [
                {
                    "type": BlockType.RESOURCE,
                    "name": "AWS::EC2::VPC.WebVPC",
                    "path": os.path.join(test_dir, f"test.{file_ext}"),
                    "attribute_key": "CidrBlock",
                },
            ],
            "SecurityGroupIngress.0": [
                {
                    "type": BlockType.RESOURCE,
                    "name": "AWS::EC2::VPC.WebVPC",
                    "path": os.path.join(test_dir, f"test.{file_ext}"),
                    "attribute_key": "CidrBlock",
                },
            ],
        }
        web_vpc_default_sg_expected_breadcrumbs = {}

        self.compare_vertex_breadcrumbs(local_graph, web_vpc_expected_breadcrumbs, BlockType.RESOURCE, 'AWS::EC2::VPC.WebVPC')
        self.compare_vertex_breadcrumbs(local_graph, my_sg_expected_breadcrumbs, BlockType.RESOURCE, 'AWS::EC2::SecurityGroup.MySG')
        self.compare_vertex_breadcrumbs(local_graph, web_vpc_default_sg_expected_breadcrumbs, BlockType.OUTPUTS, 'WebVPCDefaultSg')

    def test_render_sub(self):
        relative_path = './resources/variable_rendering/render_sub/'
        yaml_test_dir = os.path.realpath(os.path.join(TEST_DIRNAME, relative_path, 'yaml'))
        json_test_dir = os.path.realpath(os.path.join(TEST_DIRNAME, relative_path, 'json'))
        self.validate_render_sub(yaml_test_dir, 'yaml')
        self.validate_render_sub(json_test_dir, 'json')

    def validate_render_sub(self, test_dir: str, file_ext: str):
        graph_manager = CloudformationGraphManager(db_connector=NetworkxConnector())
        local_graph, _ = graph_manager.build_graph_from_source_directory(test_dir, render_variables=True)

        company_name_expected_value = "acme"
        web_vpc_expected_cidr_block = "172.16.0.0/16"

        # Parameters
        company_name_expected_attributes = {'Default': company_name_expected_value}
        environment_expected_attributes = {'Default': None}
        # Resources
        web_vpc_expected_attributes = {'CidrBlock': web_vpc_expected_cidr_block}
        default_db_expected_attributes = {'DBName': {'Fn::Sub': 'rds-${CompanyName}-${Environment}'}}
        # Outputs
        db_endpoint_sg_expected_attributes = {'Value.Fn::Sub': "${DefaultDB.Endpoint.Address}:${DefaultDB.Endpoint.Port}"}
        web_vpc_cidr_block_expected_attributes = {'Value': web_vpc_expected_cidr_block}
        cidr_block_associations_expected_attributes = {'Value.Fn::Sub': "${WebVPC.CidrBlockAssociations}"}
        default_db_name_expected_attributes = {'Value': {'Fn::Sub': 'rds-${CompanyName}-${Environment}'}}

        self.compare_vertex_attributes(local_graph, company_name_expected_attributes, BlockType.PARAMETERS, 'CompanyName')
        self.compare_vertex_attributes(local_graph, environment_expected_attributes, BlockType.PARAMETERS, 'Environment')
        self.compare_vertex_attributes(local_graph, web_vpc_expected_attributes, BlockType.RESOURCE, 'AWS::EC2::VPC.WebVPC')
        self.compare_vertex_attributes(local_graph, default_db_expected_attributes, BlockType.RESOURCE, 'AWS::RDS::DBInstance.DefaultDB')
        self.compare_vertex_attributes(local_graph, db_endpoint_sg_expected_attributes, BlockType.OUTPUTS, 'DBEndpoint')
        self.compare_vertex_attributes(local_graph, web_vpc_cidr_block_expected_attributes, BlockType.OUTPUTS, 'WebVPCCidrBlock')
        self.compare_vertex_attributes(local_graph, cidr_block_associations_expected_attributes, BlockType.OUTPUTS, 'CidrBlockAssociations')
        self.compare_vertex_attributes(local_graph, default_db_name_expected_attributes, BlockType.OUTPUTS, 'DefaultDBName')

        # Check that config is updated as well
        self.compare_vertex_config(local_graph, web_vpc_expected_attributes, BlockType.RESOURCE, 'AWS::EC2::VPC.WebVPC')

        company_name_expected_breadcrumbs = {}
        environment_expected_breadcrumbs = {}
        web_vpc_expected_breadcrumbs = {}
        default_db_expected_breadcrumbs = {}
        db_endpoint_sg_expected_breadcrumbs = {}
        web_vpc_cidr_block_expected_breadcrumbs = {'Value': [{'type': BlockType.RESOURCE, 'name': 'AWS::EC2::VPC.WebVPC', 'path': os.path.join(test_dir, f'test.{file_ext}'), 'attribute_key': 'CidrBlock'}]}
        cidr_block_associations_expected_breadcrumbs = {}
        default_db_name_expected_breadcrumbs = {}

        self.compare_vertex_breadcrumbs(local_graph, company_name_expected_breadcrumbs, BlockType.PARAMETERS, 'CompanyName')
        self.compare_vertex_breadcrumbs(local_graph, environment_expected_breadcrumbs, BlockType.PARAMETERS, 'Environment')
        self.compare_vertex_breadcrumbs(local_graph, web_vpc_expected_breadcrumbs, BlockType.RESOURCE, 'AWS::EC2::VPC.WebVPC')
        self.compare_vertex_breadcrumbs(local_graph, default_db_expected_breadcrumbs, BlockType.RESOURCE, 'AWS::RDS::DBInstance.DefaultDB')
        self.compare_vertex_breadcrumbs(local_graph, db_endpoint_sg_expected_breadcrumbs, BlockType.OUTPUTS, 'DBEndpoint')
        self.compare_vertex_breadcrumbs(local_graph, web_vpc_cidr_block_expected_breadcrumbs, BlockType.OUTPUTS, 'WebVPCCidrBlock')
        self.compare_vertex_breadcrumbs(local_graph, cidr_block_associations_expected_breadcrumbs, BlockType.OUTPUTS, 'CidrBlockAssociations')
        self.compare_vertex_breadcrumbs(local_graph, default_db_name_expected_breadcrumbs, BlockType.OUTPUTS, 'DefaultDBName')


    def test_render_subsequent_evals(self):
        relative_path = './resources/variable_rendering/render_subsequent_evals/'
        yaml_test_dir = os.path.realpath(os.path.join(TEST_DIRNAME, relative_path, 'yaml'))
        json_test_dir = os.path.realpath(os.path.join(TEST_DIRNAME, relative_path, 'json'))
        self.validate_render_subsequent_evals(yaml_test_dir, 'yaml')
        self.validate_render_subsequent_evals(json_test_dir, 'json')

    def validate_render_subsequent_evals(self, test_dir: str, file_ext: str):
        graph_manager = CloudformationGraphManager(db_connector=NetworkxConnector())
        local_graph, _ = graph_manager.build_graph_from_source_directory(test_dir, render_variables=True)

        cidr_block_expected_expected_value = "172.16.0.0/16"

        cidr_block_expected_attributes = {'Default': cidr_block_expected_expected_value}
        web_vpc_expected_attributes = {'CidrBlock': cidr_block_expected_expected_value}
        my_sg_expected_attributes = {'SecurityGroupIngress.CidrIp': cidr_block_expected_expected_value}

        self.compare_vertex_attributes(local_graph, cidr_block_expected_attributes, BlockType.PARAMETERS, 'CidrBlock')
        self.compare_vertex_attributes(local_graph, web_vpc_expected_attributes, BlockType.RESOURCE, 'AWS::EC2::VPC.WebVPC')
        self.compare_vertex_attributes(local_graph, my_sg_expected_attributes, BlockType.RESOURCE, 'AWS::EC2::SecurityGroup.MySG')

        cidr_block_expected_breadcrumbs = {}
        web_vpc_expected_breadcrumbs = {'CidrBlock': [{'type': BlockType.PARAMETERS, 'name': 'CidrBlock', 'path': os.path.join(test_dir, f'test.{file_ext}'), 'attribute_key': 'Default'}, {'type': BlockType.RESOURCE, 'name': 'AWS::EC2::VPC.WebVPC', 'path': os.path.join(test_dir, f'test.{file_ext}'), 'attribute_key': 'CidrBlock'}]}
        my_sg_expected_breadcrumbs = {
            "SecurityGroupIngress.0.CidrIp": [
                {
                    "type": BlockType.PARAMETERS,
                    "name": "CidrBlock",
                    "path": os.path.join(test_dir, f"test.{file_ext}"),
                    "attribute_key": "Default",
                },
                {
                    "type": BlockType.RESOURCE,
                    "name": "AWS::EC2::VPC.WebVPC",
                    "path": os.path.join(test_dir, f"test.{file_ext}"),
                    "attribute_key": "CidrBlock",
                },
            ],
            "SecurityGroupIngress.0": [
                {
                    "type": BlockType.PARAMETERS,
                    "name": "CidrBlock",
                    "path": os.path.join(test_dir, f"test.{file_ext}"),
                    "attribute_key": "Default",
                }, 
                {
                    "type": BlockType.RESOURCE,
                    "name": "AWS::EC2::VPC.WebVPC",
                    "path": os.path.join(test_dir, f"test.{file_ext}"),
                    "attribute_key": "CidrBlock",
                },
            ],
        }

        self.compare_vertex_breadcrumbs(local_graph, cidr_block_expected_breadcrumbs, BlockType.PARAMETERS, 'CidrBlock')
        self.compare_vertex_breadcrumbs(local_graph, web_vpc_expected_breadcrumbs, BlockType.RESOURCE, 'AWS::EC2::VPC.WebVPC')
        self.compare_vertex_breadcrumbs(local_graph, my_sg_expected_breadcrumbs, BlockType.RESOURCE, 'AWS::EC2::SecurityGroup.MySG')

    def test_render_select(self):
        relative_path = './resources/variable_rendering/render_select/'
        yaml_test_dir = os.path.realpath(os.path.join(TEST_DIRNAME, relative_path, 'yaml'))
        json_test_dir = os.path.realpath(os.path.join(TEST_DIRNAME, relative_path, 'json'))
        self.validate_render_select(yaml_test_dir, 'yaml')
        self.validate_render_select(json_test_dir, 'json')

    def validate_render_select(self, test_dir: str, file_ext: str):
        graph_manager = CloudformationGraphManager(db_connector=NetworkxConnector())
        local_graph, _ = graph_manager.build_graph_from_source_directory(test_dir, render_variables=True)

        subnet0_expected_attributes = {'CidrBlock': '10.0.48.0/24'}
        grapes_select_expected_attributes = {'Value': 'grapes'}
        out_of_bound_select_expected_attributes = {'Value.Fn::Select': ['7', ['apples', 'grapes', 'oranges', 'mangoes']]}

        self.compare_vertex_attributes(local_graph, subnet0_expected_attributes, BlockType.RESOURCE, 'AWS::EC2::Subnet.Subnet0')
        self.compare_vertex_attributes(local_graph, grapes_select_expected_attributes, BlockType.OUTPUTS, 'GrapesSelect')
        self.compare_vertex_attributes(local_graph, out_of_bound_select_expected_attributes, BlockType.OUTPUTS, 'OutOfBoundSelect')

        subnet0_expected_breadcrumbs = {'CidrBlock.Fn::Select.1': [{'type': BlockType.PARAMETERS, 'name': 'DbSubnetIpBlocks', 'path': os.path.join(test_dir, f'test.{file_ext}'), 'attribute_key': 'Default'}], 'CidrBlock.Fn::Select': [{'type': BlockType.PARAMETERS, 'name': 'DbSubnetIpBlocks', 'path': os.path.join(test_dir, f'test.{file_ext}'), 'attribute_key': 'Default'}]}
        grapes_select_expected_breadcrumbs = {}
        out_of_bound_select_expected_breadcrumbs = {}

        self.compare_vertex_breadcrumbs(local_graph, subnet0_expected_breadcrumbs, BlockType.RESOURCE, 'AWS::EC2::Subnet.Subnet0')
        self.compare_vertex_breadcrumbs(local_graph, grapes_select_expected_breadcrumbs, BlockType.OUTPUTS, 'GrapesSelect')
        self.compare_vertex_breadcrumbs(local_graph, out_of_bound_select_expected_breadcrumbs, BlockType.OUTPUTS, 'OutOfBoundSelect')

    def test_render_join(self):
        relative_path = './resources/variable_rendering/render_join/'
        yaml_test_dir = os.path.realpath(os.path.join(TEST_DIRNAME, relative_path, 'yaml'))
        json_test_dir = os.path.realpath(os.path.join(TEST_DIRNAME, relative_path, 'json'))
        self.validate_render_join(yaml_test_dir, 'yaml')
        self.validate_render_join(json_test_dir, 'json')

    def validate_render_join(self, test_dir: str, file_ext: str):
        graph_manager = CloudformationGraphManager(db_connector=NetworkxConnector())
        local_graph, _ = graph_manager.build_graph_from_source_directory(test_dir, render_variables=True)

        s3bucket1_expected_attributes = {'BucketName': 'a:b:c'}
        s3bucket2_expected_attributes = {'BucketName': 'my_bucket_name_test'}

        self.compare_vertex_attributes(local_graph, s3bucket1_expected_attributes, BlockType.RESOURCE, 'AWS::S3::Bucket.S3Bucket1')
        self.compare_vertex_attributes(local_graph, s3bucket2_expected_attributes, BlockType.RESOURCE, 'AWS::S3::Bucket.S3Bucket2')

        s3bucket1_expected_breadcrumbs = {}
        s3bucket2_expected_breadcrumbs = {'BucketName.Fn::Join.1.0': [{'type': BlockType.PARAMETERS, 'name': 'BucketName', 'path': os.path.join(test_dir, f'test.{file_ext}'), 'attribute_key': 'Default'}], 'BucketName.Fn::Join.1': [{'type': BlockType.PARAMETERS, 'name': 'BucketName', 'path': os.path.join(test_dir, f'test.{file_ext}'), 'attribute_key': 'Default'}], 'BucketName.Fn::Join': [{'type': BlockType.PARAMETERS, 'name': 'BucketName', 'path': os.path.join(test_dir, f'test.{file_ext}'), 'attribute_key': 'Default'}]}

        self.compare_vertex_breadcrumbs(local_graph, s3bucket1_expected_breadcrumbs, BlockType.RESOURCE, 'AWS::S3::Bucket.S3Bucket1')
        self.compare_vertex_breadcrumbs(local_graph, s3bucket2_expected_breadcrumbs, BlockType.RESOURCE, 'AWS::S3::Bucket.S3Bucket2')

    def test_render_if(self):
        relative_path = './resources/variable_rendering/render_if/'
        yaml_test_dir = os.path.realpath(os.path.join(TEST_DIRNAME, relative_path, 'yaml'))
        json_test_dir = os.path.realpath(os.path.join(TEST_DIRNAME, relative_path, 'json'))
        self.valiate_render_if(yaml_test_dir, 'yaml')
        self.valiate_render_if(json_test_dir, 'json')

    def valiate_render_if(self, test_dir: str, file_ext: str):
        graph_manager = CloudformationGraphManager(db_connector=NetworkxConnector())
        local_graph, _ = graph_manager.build_graph_from_source_directory(test_dir, render_variables=True)

        ec2instance_expected_attributes = {'InstanceType': 'm1.large'}
        s3bucketsuspended_expected_attributes = {'VersioningConfiguration.Status': 'Suspended'}
        s3bucketenabled_expected_attributes = {'VersioningConfiguration.Status': 'Enabled'}

        self.compare_vertex_attributes(local_graph, ec2instance_expected_attributes, BlockType.RESOURCE, 'AWS::EC2::Instance.EC2Instance')
        self.compare_vertex_attributes(local_graph, s3bucketsuspended_expected_attributes, BlockType.RESOURCE, 'AWS::S3::Bucket.S3BucketSuspended')
        self.compare_vertex_attributes(local_graph, s3bucketenabled_expected_attributes, BlockType.RESOURCE, 'AWS::S3::Bucket.S3BucketEnabled')

        instancesize_breadcrumb = {'type': BlockType.PARAMETERS, 'name': 'InstanceSize', 'path': os.path.join(test_dir, f'test.{file_ext}'), 'attribute_key': 'Default'}
        ec2instance_expected_breadcrumbs = {
            'InstanceType.Fn::If.2.Fn::If.1': [instancesize_breadcrumb],
            'InstanceType.Fn::If.2.Fn::If': [instancesize_breadcrumb],
            'InstanceType.Fn::If.2': [instancesize_breadcrumb],
            'InstanceType.Fn::If': [instancesize_breadcrumb],
            'InstanceType': [instancesize_breadcrumb],
        }
        s3bucketsuspended_expected_breadcrumbs = {}
        s3bucketenabled_expected_breadcrumbs = {}

        self.compare_vertex_breadcrumbs(local_graph, ec2instance_expected_breadcrumbs, BlockType.RESOURCE, 'AWS::EC2::Instance.EC2Instance')
        self.compare_vertex_breadcrumbs(local_graph, s3bucketsuspended_expected_breadcrumbs, BlockType.RESOURCE, 'AWS::S3::Bucket.S3BucketSuspended')
        self.compare_vertex_breadcrumbs(local_graph, s3bucketenabled_expected_breadcrumbs, BlockType.RESOURCE, 'AWS::S3::Bucket.S3BucketEnabled')

    def compare_vertex_attributes(self, local_graph, expected_attributes, block_type, block_name):
        vertex = local_graph.vertices[local_graph.vertices_block_name_map[block_type][block_name][0]]
        vertex_attributes = vertex.get_attribute_dict()
        for attribute_key, expected_value in expected_attributes.items():
            actual_value = vertex_attributes.get(attribute_key)
            if not isinstance(expected_value, dict):
                self.assertEqual(expected_value, actual_value, f'error during comparing {block_type} in attribute key: {attribute_key}')
            else:
                for cfn_func, evaluated_value in expected_value.items():
                    self.assertIn(cfn_func, actual_value, f'error during comparing {block_type} in attribute key: {attribute_key}')
                    self.assertIn(actual_value[cfn_func], evaluated_value, f'error during comparing {block_type} in attribute key: {attribute_key}')

    def compare_vertex_breadcrumbs(self, local_graph, expected_breadcrumbs, block_type, block_name):
        vertex = local_graph.vertices[local_graph.vertices_block_name_map[block_type][block_name][0]]
        vertex_breadcrumbs = vertex.breadcrumbs
        self.assertEqual(len(vertex_breadcrumbs), len(expected_breadcrumbs))
        if len(expected_breadcrumbs) > 0:
            for vertex_id, expected_value in expected_breadcrumbs.items():
                actual_value = vertex_breadcrumbs.get(vertex_id)
                self.assertEqual(expected_value, actual_value, f'actual breadcrumbs of vertex {vertex.id} different from'
                                                               f' expected. expected = {expected_breadcrumbs}'
                                                               f' and actual = {actual_value}')

    def compare_vertex_config(self, local_graph, expected_config, block_type, block_name):
        vertex = local_graph.vertices[local_graph.vertices_block_name_map[block_type][block_name][0]]
        vertex_config = vertex.config
        for attribute_key, expected_value in expected_config.items():
            # For attributes, we often test against flattened keys like "SecurityGroupIngress.CidrIp"
            # For config, which is the original nested structure (Properties), we need to traverse
            if "." in attribute_key:
                jsonpath_expression = get_jsonpath_from_evaluated_key(attribute_key)
                match = jsonpath_expression.find(vertex_config)
                if match:
                    actual_value = match[0].value
                else:
                    actual_value = None
            else:
                actual_value = vertex_config.get(attribute_key)

            if not isinstance(expected_value, dict):
                self.assertEqual(expected_value, actual_value, f'error during comparing {block_type} in config key: {attribute_key}')
            else:
                if actual_value is None:
                     self.fail(f'Config key {attribute_key} not found')
                for cfn_func, evaluated_value in expected_value.items():
                    self.assertIn(cfn_func, actual_value, f'error during comparing {block_type} in config key: {attribute_key}')
                    self.assertIn(actual_value[cfn_func], evaluated_value, f'error during comparing {block_type} in config key: {attribute_key}')



from unittest.mock import MagicMock

from pytest_mock import MockerFixture

from checkov.kustomize.utils import get_kustomize_version, get_kubectl_version


def test_get_kubectl_version_v1_27(mocker: MockerFixture):
    # given
    subprocess_mock = MagicMock()
    subprocess_mock.stdout = b'Client Version: version.Info{Major:"1", Minor:"27", GitVersion:"v1.27.2", GitCommit:"7f6f68fdabc4df88cfea2dcf9a19b2b830f1e647", GitTreeState:"clean", BuildDate:"2023-05-17T14:20:07Z", GoVersion:"go1.20.4", Compiler:"gc", Platform:"darwin/amd64"}\nKustomize Version: v5.0.1\n'

    mocker.patch("checkov.kustomize.utils.subprocess.run", return_value=subprocess_mock)

    # when
    version = get_kubectl_version(kubectl_command="kubectl")

    # then
    assert version == 1.27


def test_get_kubectl_version_v1_28(mocker: MockerFixture):
    # given
    subprocess_mock = MagicMock()
    subprocess_mock.stdout = b"Client Version: v1.28.0\nKustomize Version: v5.0.4-0.20230601165947-6ce0bf390ce3\n"

    mocker.patch("checkov.kustomize.utils.subprocess.run", return_value=subprocess_mock)

    # when
    version = get_kubectl_version(kubectl_command="kubectl")

    # then
    assert version == 1.28


def test_get_kubectl_version_none(mocker: MockerFixture):
    # given
    subprocess_mock = MagicMock()
    subprocess_mock.stdout = b"command not found: kubectl\n"

    mocker.patch("checkov.kustomize.utils.subprocess.run", return_value=subprocess_mock)

    # when
    version = get_kubectl_version(kubectl_command="kubectl")

    # then
    assert version is None


def test_get_kustomize_version_v4(mocker: MockerFixture):
    # given
    subprocess_mock = MagicMock()
    subprocess_mock.stdout = b"{Version:kustomize/v4.5.7 GitCommit:56d82a8378dfc8dc3b3b1085e5a6e67b82966bd7 BuildDate:2022-08-02T16:28:01Z GoOs:darwin GoArch:amd64}\n"

    mocker.patch("checkov.kustomize.utils.subprocess.run", return_value=subprocess_mock)

    # when
    version = get_kustomize_version(kustomize_command="kustomize")

    # then
    assert version == "v4.5.7"


def test_get_kustomize_version_v5(mocker: MockerFixture):
    # given
    subprocess_mock = MagicMock()
    subprocess_mock.stdout = b"v5.0.0\n"

    mocker.patch("checkov.kustomize.utils.subprocess.run", return_value=subprocess_mock)

    # when
    version = get_kustomize_version(kustomize_command="kustomize")

    # then
    assert version == "v5.0.0"


def test_get_kustomize_version_none(mocker: MockerFixture):
    # given
    subprocess_mock = MagicMock()
    subprocess_mock.stdout = b"command not found: kustomize\n"

    mocker.patch("checkov.kustomize.utils.subprocess.run", return_value=subprocess_mock)

    # when
    version = get_kustomize_version(kustomize_command="kustomize")

    # then
    assert version is None



import unittest
import hcl2

from checkov.terraform.checks.resource.aws.ECRRepositoryEncrypted import check
from checkov.common.models.enums import CheckResult


class TestECRRepositoryEncrypted(unittest.TestCase):

    def test_failure(self):
        hcl_res = hcl2.loads("""
                resource "aws_ecr_repository" "foo" {
                  name                 = "bar"
                  image_tag_mutability = "MUTABLE"
                
                  image_scanning_configuration {
                    scan_on_push = true
                  }
                }
        """)
        resource_conf = hcl_res['resource'][0]['aws_ecr_repository']['foo']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_success(self):
        hcl_res = hcl2.loads("""
                resource "aws_ecr_repository" "foo" {
                  name                 = "bar"
                  image_tag_mutability = "MUTABLE"

                  image_scanning_configuration {
                    scan_on_push = true
                  }
                  
                  encryption_configuration {
                    encryption_type = "KMS"
                  }
                }
        """)
        resource_conf = hcl_res['resource'][0]['aws_ecr_repository']['foo']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)


if __name__ == '__main__':
    unittest.main()



from checkov.common.models.enums import CheckResult
from checkov.terraform.checks.module.base_module_check import BaseModuleCheck


class ModuleCheck(BaseModuleCheck):
    def __init__(self):
        name = "Some test for module calls"
        id = "CKV_M_999"
        supported_resources = ['module']
        categories = []
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_module_conf(self, conf):
        """
        Some test for module source
        :param conf: module call
        :return: <CheckResult>
        """
        return CheckResult.PASSED if 'source' in conf.keys() else CheckResult.FAILED


scanner = ModuleCheck()



from aws_cdk import core
from aws_cdk import aws_redshift as redshift

class MyRedshiftClusterStack(core.Stack):
    def __init__(self, scope: core.Construct, id: str, **kwargs) -> None:
        super().__init__(scope, id, **kwargs)

        # Define Redshift Cluster with a specific Cluster Subnet Group name
        redshift.CfnCluster(
            self, 'MyRedshiftCluster',
            cluster_subnet_group_name='my-redshift-subnet-group',  # Replace with your Cluster Subnet Group name
            # Other properties for your Redshift Cluster
        )

app = core.App()
MyRedshiftClusterStack(app, "MyRedshiftClusterStack")
app.synth()



from __future__ import annotations

import logging
from pathlib import Path
from unittest.mock import patch

import pytest

from checkov.common.external_checks.verification.sources_registry import (
    is_verification_active,
    reset_for_tests,
)
from checkov.main import Checkov


@pytest.fixture(autouse=True)
def _reset_registry():
    reset_for_tests()
    try:
        yield
    finally:
        reset_for_tests()


def _patch_git_getter(returns_path: Path):
    """Patch ``checkov.main.GitGetter`` so ``.get()`` returns ``returns_path``.

    Use as a context manager wrapping the call to ``get_external_checks_dir``.
    """
    class _StubGetter:
        def __init__(self, url: str) -> None:
            self.url = url

        def get(self) -> str:
            return str(returns_path)

    return patch("checkov.main.GitGetter", _StubGetter)


def _run_chokepoint(
    checkov: Checkov, clone_result: Path, caplog=None,
) -> "tuple[list[str] | SystemExit, str]":
    """Drive the chokepoint with a patched GitGetter; return ``(result, log_text)``.

    ``result`` is the returned dir list on success, or the ``SystemExit``
    on verification failure. ``log_text`` is captured via ``caplog`` if
    provided; otherwise an empty string.
    """
    log_capture = (
        caplog.at_level(logging.ERROR, logger="checkov.main")
        if caplog is not None
        else _noop_ctx()
    )
    # atexit.register inside main.py is harmless under test (the temp dir
    # is owned by the test's tmp_path), but mute it to keep test output clean.
    with patch("checkov.main.bc_integration") as bc, \
         patch("checkov.main.atexit"), \
         _patch_git_getter(clone_result), \
         log_capture:
        bc.sast_custom_policies = None
        try:
            result = checkov.get_external_checks_dir()
        except SystemExit as exc:
            return exc, (caplog.text if caplog is not None else "")
    return result, (caplog.text if caplog is not None else "")


import contextlib as _contextlib


@_contextlib.contextmanager
def _noop_ctx():
    yield


# --------------------------------------------------------------------------
# Happy path
# --------------------------------------------------------------------------


def test_git_no_key_skips_verification(valid_dir: Path, make_checkov):
    """``--external-checks-git`` without a public key → no verification at all."""
    checkov = make_checkov(
        external_checks_git=["https://example.invalid/repo.git"],
    )
    result, _ = _run_chokepoint(checkov, valid_dir)
    assert isinstance(result, list)
    assert result == [str(valid_dir)]
    assert is_verification_active() is False


def test_git_signed_clone_with_matching_key_verifies(
    valid_dir: Path, key_a_pub_pem: bytes, tmp_path: Path, make_checkov,
):
    """A signed git clone + matching key → verification succeeds, registry active."""
    key_path = tmp_path / "key.pem"
    key_path.write_bytes(key_a_pub_pem)

    checkov = make_checkov(
        external_checks_git=["https://example.invalid/repo.git"],
        external_checks_public_key=[str(key_path)],
    )
    result, _ = _run_chokepoint(checkov, valid_dir)
    assert isinstance(result, list)
    assert result == [str(valid_dir)]
    assert is_verification_active() is True


# --------------------------------------------------------------------------
# Rejection paths
# --------------------------------------------------------------------------


def test_git_unsigned_clone_with_key_exits_via_exit_run(
    unsigned_dir: Path, key_a_pub_pem: bytes, tmp_path: Path, make_checkov, caplog,
):
    """An unsigned git clone + key → exit_run() invoked, no scan.

    Routing through ``self.exit_run()`` is what gives ``--no-fail-on-crash``
    its documented contract — exit 0 with the failure still surfaced on
    stderr (via logging).
    """
    key_path = tmp_path / "key.pem"
    key_path.write_bytes(key_a_pub_pem)

    checkov = make_checkov(
        external_checks_git=["https://example.invalid/repo.git"],
        external_checks_public_key=[str(key_path)],
    )

    result, log_text = _run_chokepoint(checkov, unsigned_dir, caplog=caplog)

    assert isinstance(result, SystemExit)
    assert result.code == 2
    assert "External checks signature verification failed" in log_text
    assert is_verification_active() is False


def test_git_modified_clone_with_key_exits_2(
    mutated_dir: Path, key_a_pub_pem: bytes, tmp_path: Path, make_checkov, caplog,
):
    """A signed-then-mutated git clone + key → exit 2 + diagnostic logged."""
    key_path = tmp_path / "key.pem"
    key_path.write_bytes(key_a_pub_pem)

    checkov = make_checkov(
        external_checks_git=["https://example.invalid/repo.git"],
        external_checks_public_key=[str(key_path)],
    )

    result, log_text = _run_chokepoint(checkov, mutated_dir, caplog=caplog)

    assert isinstance(result, SystemExit)
    assert result.code == 2
    assert "External checks signature verification failed" in log_text


def test_git_unsigned_clone_with_key_and_no_fail_on_crash_exits_0(
    unsigned_dir: Path, key_a_pub_pem: bytes, tmp_path: Path, make_checkov, caplog,
):
    """Same as the unsigned-rejection test, but with ``--no-fail-on-crash``.

    Documents that the flag's "exit 0 instead of 2" contract applies to
    git-sourced verification failures exactly the same as to local-dir
    verification failures.
    """
    key_path = tmp_path / "key.pem"
    key_path.write_bytes(key_a_pub_pem)

    checkov = make_checkov(
        external_checks_git=["https://example.invalid/repo.git"],
        external_checks_public_key=[str(key_path)],
        no_fail_on_crash=True,
    )

    result, log_text = _run_chokepoint(checkov, unsigned_dir, caplog=caplog)

    assert isinstance(result, SystemExit)
    assert result.code == 0
    # Diagnostic surface is unchanged — only the exit code differs.
    assert "External checks signature verification failed" in log_text


# --------------------------------------------------------------------------
# Flag interaction
# --------------------------------------------------------------------------


def test_git_overrides_local_dir_when_both_are_set(
    valid_dir: Path, key_a_pub_pem: bytes, tmp_path: Path, make_checkov,
):
    """If both ``--external-checks-dir`` AND ``--external-checks-git`` are passed,
    the git clone REPLACES the local dir.

    Documents the current chokepoint behaviour at
    [`main.py:802-805`](checkov/checkov/main.py:802) — there is no CLI-level
    mutex enforcement, so this assignment matters. Verification then runs
    against the git clone path only.

    If a future change adds CLI-level mutex enforcement, this test should
    be updated to assert the CLI rejects the combination instead.
    """
    # The "local dir" is a throwaway sibling that should NOT be verified.
    decoy_dir = tmp_path / "decoy_local_dir_that_should_be_ignored"
    decoy_dir.mkdir()
    (decoy_dir / "__init__.py").write_text("")
    (decoy_dir / "should_not_be_loaded.py").write_text(
        "# unsigned decoy — must not appear in the verified allowlist\n"
    )

    key_path = tmp_path / "key.pem"
    key_path.write_bytes(key_a_pub_pem)

    checkov = make_checkov(
        external_checks_dir=[str(decoy_dir)],
        external_checks_git=["https://example.invalid/repo.git"],
        external_checks_public_key=[str(key_path)],
    )
    # The "clone" returns the signed valid_dir, so verification should succeed.
    result, _ = _run_chokepoint(checkov, valid_dir)
    assert isinstance(result, list)
    # The returned list is the git clone path, NOT the decoy local dir.
    assert result == [str(valid_dir)]
    assert str(decoy_dir) not in result, (
        "the local --external-checks-dir argument should have been overwritten "
        "by the git clone result; the decoy dir leaked through"
    )
    assert is_verification_active() is True


# --------------------------------------------------------------------------
# Robustness: clone artefacts inside the temp dir
# --------------------------------------------------------------------------


def test_git_clone_with_pycache_in_repo_is_handled(
    tmp_path: Path, priv_a, key_a_pub_pem: bytes, make_trailer, make_checkov,
):
    """A git clone that happens to ship a ``__pycache__/`` (e.g. checked in by
    accident or left over from local CI) must be silently skipped by the
    walker — same behaviour as the local-dir case.
    """
    clone = tmp_path / "fake_clone"
    clone.mkdir()
    (clone / "__init__.py").write_bytes(make_trailer(b"", priv_a))
    body = b"# signed via git\nCHECK_ID = 'CKV_GIT_1'\n"
    (clone / "checked.py").write_bytes(make_trailer(body, priv_a))
    # And a pre-existing __pycache__/ with a stale .pyc — the walker
    # skips __pycache__ subdirs entirely.
    cache_dir = clone / "__pycache__"
    cache_dir.mkdir()
    (cache_dir / "stale.cpython-312.pyc").write_bytes(b"\x00fake bytecode")

    key_path = tmp_path / "key.pem"
    key_path.write_bytes(key_a_pub_pem)

    checkov = make_checkov(
        external_checks_git=["https://example.invalid/repo.git"],
        external_checks_public_key=[str(key_path)],
    )
    result, _ = _run_chokepoint(checkov, clone)
    assert isinstance(result, list)
    assert is_verification_active() is True


# --------------------------------------------------------------------------
# Multiple --external-checks-git args
# --------------------------------------------------------------------------


def test_git_only_first_url_is_used(
    valid_dir: Path, key_a_pub_pem: bytes, tmp_path: Path, make_checkov,
):
    """``--external-checks-git`` is declared as ``action='append'`` but the
    chokepoint only consumes the first URL — confirm so future operators
    don't silently lose verification of the second/third URL.

    Pins the current behaviour at
    [`main.py:804`](checkov/checkov/main.py:804) — ``self.config.external_checks_git[0]``.
    """
    key_path = tmp_path / "key.pem"
    key_path.write_bytes(key_a_pub_pem)

    checkov = make_checkov(
        external_checks_git=[
            "https://example.invalid/first.git",
            "https://example.invalid/second.git",  # SILENTLY IGNORED today
        ],
        external_checks_public_key=[str(key_path)],
    )
    captured_urls: list[str] = []

    class _RecordingGetter:
        def __init__(self, url: str) -> None:
            captured_urls.append(url)

        def get(self) -> str:
            return str(valid_dir)

    with patch("checkov.main.GitGetter", _RecordingGetter), \
         patch("checkov.main.bc_integration") as bc, \
         patch("checkov.main.atexit"):
        bc.sast_custom_policies = None
        result = checkov.get_external_checks_dir()

    assert isinstance(result, list)
    assert captured_urls == ["https://example.invalid/first.git"], (
        "the chokepoint should consume only the first --external-checks-git URL; "
        f"got URLs={captured_urls!r}. If this fails, the chokepoint was changed to "
        "iterate all URLs — update the test, and also verify each URL's clone runs "
        "through verify_and_register."
    )


# --------------------------------------------------------------------------
# S4: TOCTOU regression — late mutation of git clone caught by loader escalation
# --------------------------------------------------------------------------


def test_late_modification_of_git_clone_is_caught_by_loader_escalation(
    valid_dir: Path, key_a_pub_pem: bytes, tmp_path: Path, make_checkov, stub_registry,
):
    """Closes the S4 disk-drift window: between ``verify_and_register`` (at
    the chokepoint) and ``load_external_checks`` (during the scan), the
    cloned git directory lives on disk and is mutable. A careless
    build-script artefact (or any unrelated process) dropping an extra
    ``.py`` into the cloned dir AFTER verification but BEFORE the loader
    walks it must NOT silently shrink/grow the verified check set — the
    loader's M1+S3 escalation must catch the divergence and raise
    ``SignatureVerificationError``.

    Mirrors the local-dir regression already pinned in
    ``test_verified_loader.test_load_external_checks_refuses_unverified_file``,
    but starts from the git-clone code path so the regression covers the
    full ``--external-checks-git`` lifecycle end-to-end. No production-code
    change is required if the existing M1+S3 fix is wired correctly.
    """
    from checkov.common.external_checks.verification import (
        SignatureVerificationError,
    )

    key_path = tmp_path / "key.pem"
    key_path.write_bytes(key_a_pub_pem)

    checkov = make_checkov(
        external_checks_git=["https://example.invalid/repo.git"],
        external_checks_public_key=[str(key_path)],
    )

    # Chokepoint runs against the signed ``valid_dir`` (stand-in for the
    # post-clone temp dir GitGetter would have returned in production).
    result, _ = _run_chokepoint(checkov, valid_dir)
    assert isinstance(result, list)
    assert is_verification_active() is True

    # Simulate the TOCTOU window: drop an unverified file into the cloned
    # dir AFTER verify_and_register has populated the in-memory allowlist.
    # Bytes that would raise on exec — proves the loader never exec'd them
    # (the divergence is caught at the resolve step, before exec).
    evil = valid_dir / "evil.py"
    evil.write_bytes(b"raise RuntimeError('TOCTOU drop was executed')\n")

    registry = stub_registry

    # The loader resolves ``verified_sources`` from the in-memory registry
    # populated by the chokepoint, then walks the dir; ``evil.py`` is on
    # disk but absent from the allowlist → escalates.
    with pytest.raises(SignatureVerificationError) as exc:
        registry.load_external_checks(str(valid_dir.resolve()))

    assert "evil.py" in str(exc.value)



from aws_cdk import core
from aws_cdk import aws_elasticache as elasticache

class MyElastiCacheReplicationGroupStack(core.Stack):
    def __init__(self, scope: core.Construct, id: str, **kwargs) -> None:
        super().__init__(scope, id, **kwargs)

        # Define ElastiCache Replication Group with encryption and auth token
        elasticache.CfnReplicationGroup(
            self, 'MyElastiCacheReplicationGroup',
            replication_group_description='MyReplicationGroup',
            cache_node_type='cache.t2.small',
            engine='redis',
            engine_version='6.x',
            num_node_groups=1,
            automatic_failover_enabled=True,
            transit_encryption_enabled=True,  # Enable transit encryption
            auth_token='YourAuthTokenHere'  # Provide the auth token
            # ... other properties as needed
        )

app = core.App()
MyElastiCacheReplicationGroupStack(app, "MyElastiCacheReplicationGroupStack")
app.synth()



from unittest import mock

import pytest
from checkov.common.images.image_referencer import Image
from checkov.terraform.image_referencer.manager import TerraformImageReferencerManager
from tests.graph_utils.utils import set_graph_by_graph_framework, add_vertices_to_graph_by_graph_framework, \
    GRAPH_FRAMEWORKS


@pytest.mark.parametrize("graph_framework", GRAPH_FRAMEWORKS)
def test_extract_images_from_resources(graph_framework):
    # given
    aws_resource = {
        "file_path_": "/ecs.tf",
        "__end_line__": 31,
        "__start_line__": 1,
        "container_definitions": [
            {
                "name": "first",
                "image": "nginx",
                "cpu": 10,
                "memory": 512,
                "essential": True,
                "portMappings": [{"containerPort": 80, "hostPort": 80}],
            },
        ],
        "resource_type": "aws_ecs_task_definition",
    }
    azure_resource = {
        "file_path_": "/batch.tf",
        "__end_line__": 25,
        "__start_line__": 1,
        "container_configuration": {
            "container_image_names": ["python:3.9-alpine"],
            "container_registries": {
                "password": "myPassword",  # checkov:skip=CKV_SECRET_6 test secret
                "registry_server": "myContainerRegistry.azurecr.io",
                "user_name": "myUserName",
            },
            "type": "DockerCompatible",
        },
        "resource_type": "azurerm_batch_pool",
    }
    gcp_resource = {
        "file_path_": "/cloud_run.tf",
        "__end_line__": 17,
        "__start_line__": 1,
        "template": {
            "spec": {
                "containers": {
                    "image": "gcr.io/cloudrun/hello",
                }
            }
        },
        "resource_type": "google_cloud_run_service",
    }
    graph = set_graph_by_graph_framework(graph_framework)
    add_vertices_to_graph_by_graph_framework(graph_framework, aws_resource, graph, 1, 'first')
    add_vertices_to_graph_by_graph_framework(graph_framework, azure_resource, graph, 2, '2')
    add_vertices_to_graph_by_graph_framework(graph_framework, gcp_resource, graph, 3, '3')

    # when
    with mock.patch.dict('os.environ', {'CHECKOV_GRAPH_FRAMEWORK': graph_framework}):
        images = TerraformImageReferencerManager(graph_connector=graph).extract_images_from_resources()

    # then
    assert images == [
        Image(file_path="/ecs.tf", name="nginx", start_line=1, end_line=31, related_resource_id="/ecs.tf:None"),
        Image(
            file_path="/batch.tf",
            name="python:3.9-alpine",
            start_line=1,
            end_line=25,
            related_resource_id="/batch.tf:None",
        ),
        Image(
            file_path="/cloud_run.tf",
            name="gcr.io/cloudrun/hello",
            start_line=1,
            end_line=17,
            related_resource_id="/cloud_run.tf:None",
        ),
    ]




from aws_cdk import core
from aws_cdk import aws_ecr as ecr

class MyECRRepositoryStack(core.Stack):
    def __init__(self, scope: core.Construct, id: str, **kwargs) -> None:
        super().__init__(scope, id, **kwargs)

        # Define ECR Repository with Encryption Configuration
        ecr.CfnRepository(
            self, 'MyECRRepository',
            repository_name='my-ecr-repo',
            encryption_configuration={
                'encryptionType': 'KMS'
            }
            # Other properties for your ECR Repository
        )

app = core.App()
MyECRRepositoryStack(app, "MyECRRepositoryStack")
app.synth()



import unittest

from checkov.common.models.enums import CheckResult
from checkov.terraform.checks.resource.aws.RedshiftClusterKMSKey import check
import hcl2


class TestRedshiftClusterKMSKey(unittest.TestCase):

    def test_failure(self):
        hcl_res = hcl2.loads("""
                resource "aws_redshift_cluster" "test" {
                  cluster_identifier = "tf-redshift-cluster"
                  database_name      = "mydb"
                  master_username    = "foo"
                  master_password    = "Mustbe8characters"  # checkov:skip=CKV_SECRET_6 test secret
                  node_type          = "dc1.large"
                  cluster_type       = "single-node"
                }
            """)
        resource_conf = hcl_res['resource'][0]['aws_redshift_cluster']['test']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_success(self):
        hcl_res = hcl2.loads("""
                resource "aws_redshift_cluster" "test" {
                  cluster_identifier = "tf-redshift-cluster"
                  database_name      = "mydb"
                  master_username    = "foo"
                  master_password    = "Mustbe8characters"
                  node_type          = "dc1.large"
                  cluster_type       = "single-node"
                  kms_key_id         = "someKey"
                }
            """)
        resource_conf = hcl_res['resource'][0]['aws_redshift_cluster']['test']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)


if __name__ == "__main__":
    unittest.main()



import unittest

import hcl2

from checkov.terraform.checks.provider.aws.credentials import check
from checkov.common.models.enums import CheckResult


class TestCredentials(unittest.TestCase):
    def test_success_empty(self):
        hcl_res = hcl2.loads(
            """
            provider "aws" {}
            """
        )
        provider_conf = hcl_res["provider"][0]["aws"]
        scan_result = check.scan_provider_conf(conf=provider_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)

    def test_success_region(self):
        hcl_res = hcl2.loads(
            """
            provider "aws" {
                region = "us-west-2"
            }
            """
        )
        provider_conf = hcl_res["provider"][0]["aws"]
        scan_result = check.scan_provider_conf(conf=provider_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)

    def test_failure_both_keys(self):
        hcl_res = hcl2.loads(
            """
            provider "aws" {
                region     = "us-west-2"
                access_key = "AKIAIOSFODNN7EXAMPLE"
                secret_key = "wJalrXUtnFEMI/K7MDENG/bPxRfiCYEXAMPLEKEY"
            }
            """
        )
        provider_conf = hcl_res["provider"][0]["aws"]
        scan_result = check.scan_provider_conf(conf=provider_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_failure_access_key(self):
        hcl_res = hcl2.loads(
            """
            provider "aws" {
                region     = "us-west-2"
                access_key = "AKIAIOSFODNN7EXAMPLE"
            }
            """
        )
        provider_conf = hcl_res["provider"][0]["aws"]
        scan_result = check.scan_provider_conf(conf=provider_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_failure_secret_key(self):
        hcl_res = hcl2.loads(
            """
            provider "aws" {
                region     = "us-west-2"
                secret_key = "wJalrXUtnFEMI/K7MDENG/bPxRfiCYEXAMPLEKEY"
            }
            """
        )
        provider_conf = hcl_res["provider"][0]["aws"]
        scan_result = check.scan_provider_conf(conf=provider_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)


if __name__ == "__main__":
    unittest.main()



import os

from parameterized import parameterized_class

from tests.graph_utils.utils import PARAMETERIZED_GRAPH_FRAMEWORKS
from tests.terraform.graph.checks_infra.test_base import TestBaseSolver

TEST_DIRNAME = os.path.dirname(os.path.realpath(__file__))


@parameterized_class(PARAMETERIZED_GRAPH_FRAMEWORKS)
class TestCIDRRangeSubsetSolver(TestBaseSolver):
    def setUp(self):
        self.checks_dir = TEST_DIRNAME
        super(TestCIDRRangeSubsetSolver, self).setUp()

    def test_cidr_range_subset_string_solver(self):
        root_folder = 'resources'
        check_id = "CIDRRangeSubsetString"
        should_pass = ['test.pass1', 'test.pass2', 'test.pass3', 'test.pass4']
        should_fail = ['test.fail1', 'test.fail2', 'test.fail3']
        expected_results = {check_id: {"should_pass": should_pass, "should_fail": should_fail}}

        self.run_test(root_folder=root_folder, expected_results=expected_results, check_id=check_id)

    def test_cidr_range_subset_string_jsonpath_solver(self):
        root_folder = 'resources'
        check_id = "JsonPathCIDRRangeSubsetString"
        should_pass = ['test.pass1', 'test.pass2', 'test.pass3', 'test.pass4']
        should_fail = ['test.fail1', 'test.fail2', 'test.fail3']
        expected_results = {check_id: {"should_pass": should_pass, "should_fail": should_fail}}

        self.run_test(root_folder=root_folder, expected_results=expected_results, check_id=check_id)

    def test_cidr_range_subset_list_solver(self):
        root_folder = 'resources'
        check_id = "CIDRRangeSubsetList"
        should_pass = ['test.pass1', 'test.pass2', 'test.pass3', 'test.pass4']
        should_fail = ['test.fail1', 'test.fail2', 'test.fail3']
        expected_results = {check_id: {"should_pass": should_pass, "should_fail": should_fail}}

        self.run_test(root_folder=root_folder, expected_results=expected_results, check_id=check_id)

    def test_cidr_range_subset_list_jsonpath_solver(self):
        root_folder = 'resources'
        check_id = "JsonPathCIDRRangeSubsetList"
        should_pass = ['test.pass1', 'test.pass2', 'test.pass3', 'test.pass4']
        should_fail = ['test.fail1', 'test.fail2', 'test.fail3']
        expected_results = {check_id: {"should_pass": should_pass, "should_fail": should_fail}}

        self.run_test(root_folder=root_folder, expected_results=expected_results, check_id=check_id)

    def test_ipv6_cidr_range_subset_list_solver(self):
        root_folder = 'resources'
        check_id = "IPV6CIDRRangeSubsetList"
        should_pass = ['test.ipv6_pass1', 'test.ipv6_pass2']
        should_fail = ['test.ipv6_fail1', 'test.ipv6_fail2', 'test.pass1', 'test.pass2', 'test.pass3', 'test.pass4',
                       'test.fail1', 'test.fail2', 'test.fail3']
        expected_results = {check_id: {"should_pass": should_pass, "should_fail": should_fail}}

        self.run_test(root_folder=root_folder, expected_results=expected_results, check_id=check_id)

    def test_ipv6_cidr_range_subset_list_jsonpath_solver(self):
        root_folder = 'resources'
        check_id = "JsonPathIPV6CIDRRangeSubsetList"
        should_pass = ['test.ipv6_pass1', 'test.ipv6_pass2']
        should_fail = ['test.ipv6_fail1', 'test.ipv6_fail2', 'test.pass1', 'test.pass2', 'test.pass3', 'test.pass4',
                       'test.fail1', 'test.fail2', 'test.fail3']
        expected_results = {check_id: {"should_pass": should_pass, "should_fail": should_fail}}

        self.run_test(root_folder=root_folder, expected_results=expected_results, check_id=check_id)



from aws_cdk import core
from aws_cdk import aws_athena as athena

class AthenaStack(core.Stack):

    def __init__(self, scope: core.Construct, id: str, **kwargs) -> None:
        super().__init__(scope, id, **kwargs)

        # Create an Athena WorkGroup
        workgroup = athena.CfnWorkGroup(
            self,
            "MyAthenaWorkGroup",
            name="my-workgroup",
            description="My Athena WorkGroup",
            state="ENABLED",  # You can change the state
            work_group_configuration=athena.CfnWorkGroup.WorkGroupConfigurationProperty(
                additional_configuration="additionalConfiguration",
                bytes_scanned_cutoff_per_query=123,
                customer_content_encryption_configuration=athena.CfnWorkGroup.CustomerContentEncryptionConfigurationProperty(
                    kms_key="kmsKey"
                ),
                enforce_work_group_configuration=False,
            )
        )

app = core.App()
AthenaStack(app, "AthenaStack")
app.synth()



from typing import List

from checkov.common.models.enums import CheckCategories
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck


class AzureContainerInstancePublicIPAddressType(BaseResourceValueCheck):
    def __init__(self) -> None:
        name = "Ensure that Azure Container group is deployed into virtual network"
        id = "CKV_AZURE_245"
        supported_resources = ('azurerm_container_group',)
        categories = (CheckCategories.NETWORKING,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self) -> str:
        return 'ip_address_type'

    def get_expected_values(self) -> List[str]:
        return ['Private', 'None']


check = AzureContainerInstancePublicIPAddressType()



import unittest

import hcl2

from checkov.common.models.enums import CheckResult
from checkov.terraform.checks.resource.aws.CloudformationStackNotificationArns import check


class TestCloudformationStackNotificationArns(unittest.TestCase):

    def test_failure(self):
        hcl_res = hcl2.loads("""
                resource "aws_cloudformation_stack" "default" {
                  name = "networking-stack"
                
                  parameters = {
                    VPCCidr = "10.0.0.0/16"
                  }
                }
                """)
        resource_conf = hcl_res['resource'][0]['aws_cloudformation_stack']['default']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_success(self):
        hcl_res = hcl2.loads("""
                resource "aws_cloudformation_stack" "default" {
                  name = "networking-stack"
                
                  parameters = {
                    VPCCidr = "10.0.0.0/16"
                  }
                  notification_arns = ["arn1", "arn2"]
                }
                """)
        resource_conf = hcl_res['resource'][0]['aws_cloudformation_stack']['default']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)


if __name__ == '__main__':
    unittest.main()



import unittest

import hcl2

from checkov.terraform.checks.resource.aws.RDSDeletionProtection import check
from checkov.common.models.enums import CheckResult


class TestRDSDeletionProtection(unittest.TestCase):

    def test_failure(self):
        hcl_res = hcl2.loads("""
                resource "aws_rds_cluster" "default" {
                    cluster_identifier      = "aurora-cluster-demo"
                    availability_zones      = ["us-west-2a", "us-west-2b", "us-west-2c"]
                    database_name           = "mydb"
                    master_username         = "foo"
                    master_password         = "bar"
                    backup_retention_period = 5
                    preferred_backup_window = "07:00-09:00"
                    deletion_protection = false
                }
                """)
        resource_conf = hcl_res['resource'][0]['aws_rds_cluster']['default']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_failure_missing_attribute(self):
        hcl_res = hcl2.loads("""
                resource "aws_rds_cluster" "default" {
                    cluster_identifier      = "aurora-cluster-demo"
                    availability_zones      = ["us-west-2a", "us-west-2b", "us-west-2c"]
                    database_name           = "mydb"
                    master_username         = "foo"
                    master_password         = "bar"
                    backup_retention_period = 5
                    preferred_backup_window = "07:00-09:00"
                }
                """)
        resource_conf = hcl_res['resource'][0]['aws_rds_cluster']['default']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_success(self):
        hcl_res = hcl2.loads("""
                resource "aws_rds_cluster" "default" {
                    cluster_identifier      = "aurora-cluster-demo"
                    availability_zones      = ["us-west-2a", "us-west-2b", "us-west-2c"]
                    database_name           = "mydb"
                    master_username         = "foo"
                    master_password         = "bar"
                    backup_retention_period = 5
                    preferred_backup_window = "07:00-09:00"
                    deletion_protection = true

                }
                """)
        resource_conf = hcl_res['resource'][0]['aws_rds_cluster']['default']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)


if __name__ == '__main__':
    unittest.main()



import unittest

from checkov.common.bridgecrew.integration_features.features.policy_metadata_integration import \
    PolicyMetadataIntegration
from checkov.common.bridgecrew.platform_integration import BcPlatformIntegration



class TestPolicyMetadataIntegration(unittest.TestCase):

    def test_filtered_policy_ids(self):
        instance = BcPlatformIntegration()
        instance.bc_api_key = '00000000-0000-0000-0000-000000000000::1234=='
        instance.customer_run_config_response = mock_customer_run_config()
        instance.prisma_policies_response = mock_prisma_policies_response()
        instance.prisma_policies_exception_response = [mock_prisma_policies_response()[0]]
        metadata_integration = PolicyMetadataIntegration(instance)
        metadata_integration.bc_integration = instance
        metadata_integration.pre_scan()
        metadata_integration.pc_to_ckv_id_mapping
        self.assertDictEqual(metadata_integration.pc_to_ckv_id_mapping, {'6960be11-e3a6-46cc-bf66-933c57c2af5d': 'CKV_AWS_212', '3dc2478c-bf25-4383-aaa1-30feb5cda586': '806079891421835264_AZR_1685557908904', 'c11ce08c-b93e-4e11-8d1c-e5a1339139d1': 'CKV_AWS_40', '0e4c576e-c934-4af3-8592-a53920e71ffb': 'CKV_AWS_53', '1234': 'CKV3_SAST_123'})
        self.assertListEqual(metadata_integration.filtered_policy_ids, ['CKV_AWS_212', '806079891421835264_AZR_1685557908904', 'CKV_AWS_40', 'CKV_AWS_53', 'CKV_AZURE_122'])
        self.assertListEqual(metadata_integration.filtered_exception_policy_ids, ['CKV_AWS_212'])
        self.assertSetEqual(set(metadata_integration.sast_check_metadata.keys()), {'CKV3_SAST_123'})


def mock_customer_run_config():
    return {
        "policyMetadata": {
            "CKV_AWS_212": {
                "id": "BC_AWS_IAM_5",
                "title": "Ensure IAM password policy requires at least one uppercase letter",
                "guideline": "https://docs.bridgecrew.io/docs/iam_5",
                "severity": "MEDIUM",
                "pcSeverity": "MEDIUM",
                "category": "IAM",
                "checkovId": "CKV_AWS_212",
                "constructiveTitle": "Ensure AWS IAM password policy has an uppercase character",
                "descriptiveTitle": "AWS IAM password policy does not have an uppercase character",
                "pcPolicyId": "6960be11-e3a6-46cc-bf66-933c57c2af5d",
                "additionalPcPolicyIds": [
                    "6960be11-e3a6-46cc-bf66-933c57c2af5d"
                ],
                "benchmarks": {}
            },
            "CKV_AWS_40": {
                "id": "BC_AWS_IAM_16",
                "title": "Ensure IAM policies are attached only to groups or roles",
                "guideline": "https://docs.bridgecrew.io/docs/iam_16-iam-policy-privileges-1",
                "severity": "CRITICAL",
                "pcSeverity": "HIGH",
                "category": "IAM",
                "checkovId": "CKV_AWS_40",
                "constructiveTitle": "Ensure IAM policies are only attached to Groups and Roles",
                "descriptiveTitle": "AWS IAM policy attached to users",
                "pcPolicyId": "c11ce08c-b93e-4e11-8d1c-e5a1339139d1",
                "additionalPcPolicyIds": [
                    "c11ce08c-b93e-4e11-8d1c-e5a1339139d1"
                ],
                "benchmarks": {}
            },
            "CKV_AWS_53": {
                "id": "BC_AWS_S3_19",
                "title": "Ensure S3 bucket has block public ACLS enabled",
                "guideline": "https://docs.bridgecrew.io/docs/bc_aws_s3_19",
                "severity": "MEDIUM",
                "pcSeverity": None,
                "category": "Storage",
                "checkovId": "CKV_AWS_53",
                "constructiveTitle": "Ensure S3 bucket has block public ACLS enabled",
                "descriptiveTitle": None,
                "pcPolicyId": "0e4c576e-c934-4af3-8592-a53920e71ffb",
                "additionalPcPolicyIds": [
                    "0e4c576e-c934-4af3-8592-a53920e71ffb"
                ],
                "benchmarks": {}
            },
            "CKV_AZURE_122": {
                "id": "BC_AZR_NETWORKING_39",
                "title": "Ensure that Application Gateway uses WAF in \"Detection\" or \"Prevention\" modes",
                "guideline": "https://docs.bridgecrew.io/docs/ensure-that-application-gateway-uses-waf-in-detection-or-prevention-modes",
                "severity": "LOW",
                "pcSeverity": "LOW",
                "category": "Networking",
                "checkovId": "CKV_AZURE_122",
                "constructiveTitle": "Ensure application gateway uses WAF in Detection or Prevention modes",
                "descriptiveTitle": "Application gateway does not use WAF in Detection or Prevention modes",
                "pcPolicyId": "3dc2478c-bf25-4383-aaa1-30feb5cda586",
                "additionalPcPolicyIds": [
                    "3dc2478c-bf25-4383-aaa1-30feb5cda586"
                ],
                "benchmarks": {}
            },
            "CKV3_SAST_123": {
                "id": "BC_SAST_123",
                "title": "sast",
                "guideline": "https://docs.bridgecrew.io/docs/abc",
                "severity": "LOW",
                "pcSeverity": "LOW",
                "category": "Networking",
                "checkovId": "CKV3_SAST_123",
                "constructiveTitle": "sast",
                "descriptiveTitle": "sast",
                "pcPolicyId": "1234",
                "additionalPcPolicyIds": [
                    "1234"
                ],
                "benchmarks": {}
            }
        },
        "customPolicies": [
            {
            "id": "806079891421835264_AZR_1685557908904",
            "code": "null",
            "title": "Application gateway does not use WAF in Detection or Prevention modes",
            "guideline": "Refer the documentation for more details,\nhttps://docs.bridgecrew.io/docs/ensure-that-application-gateway-uses-waf-in-detection-or-prevention-modes",
            "severity": "MEDIUM",
            "pcSeverity": "MEDIUM",
            "category": "Networking",
            "pcPolicyId": "3dc2478c-bf25-4383-aaa1-30feb5cda586",
            "additionalPcPolicyIds": [
                "3dc2478c-bf25-4383-aaa1-30feb5cda586"
            ],
            "sourceIncidentId": "BC_AZR_NETWORKING_39",
            "benchmarks": {},
            "frameworks": [
                "CloudFormation",
                "Terraform"
            ],
            "provider": "Azure"
            }
        ]
    }


def mock_prisma_policies_response():
    return [
        {
            "policyId": "6960be11-e3a6-46cc-bf66-933c57c2af5d",
            "name": "AWS EBS volume region with encryption is disabled",
            "policyType": "config",
            "policySubTypes": [
                "run",
                "build"
            ],
            "systemDefault": True,
            "policyUpi": "PC-AWS-EC2-778",
            "description": "This policy identifies AWS regions in which new EBS volumes are getting created without any encryption. Encrypting data at rest reduces unintentional exposure of data stored in EBS volumes. It is recommended to configure EBS volume at the regional level so that every new EBS volume created in that region will be enabled with encryption by using a provided encryption key.",
            "severity": "medium",
            "rule": {
                "name": "AWS EBS volume region with encryption is disabled",
                "criteria": "7a951e9f-02d2-4d9f-9441-29b545084585",
                "parameters": {
                    "withIac": "true",
                    "savedSearch": "true"
                },
                "type": "Config",
                "children": [
                    {
                        "criteria": "{\"category\":\"Kubernetes\",\"resourceTypes\":[\"aws_ebs_encryption_by_default\"]}",
                        "type": "build",
                        "metadata": {
                            "checkovId": "CKV_AWS_106"
                        },
                        "recommendation": "Refer the documentation for more details,\nhttps://docs.bridgecrew.io/docs/ensure-kubernetes-secrets-are-encrypted-using-customer-master-keys-cmks-managed-in-aws-kms"
                    }
                ]
            },
            "recommendation": "To enable encryption at region level by default, follow below URL:\nhttps://docs.aws.amazon.com/AWSEC2/latest/UserGuide/EBSEncryption.html#encryption-by-default\n\nAdditional Information:\n\nTo detect existing EBS volumes that are not encrypted ; refer Saved Search:\nAWS EBS volumes are not encrypted_RL\n\nTo detect existing EBS volumes that are not encrypted with CMK, refer Saved Search:\nAWS EBS volume not encrypted using Customer Managed Key_RL",
            "cloudType": "aws",
            "complianceMetadata": [
                {
                    "standardId": "a0ea1077-424f-45fd-994e-4caef6d4d9de",
                    "standardName": "AWS Foundational Security Best Practices standard",
                    "standardDescription": "AWS Foundational Security Best Practices standard",
                    "requirementId": "Protect",
                    "requirementName": "Protect",
                    "sectionId": "Data protection",
                    "sectionDescription": "Data protection",
                    "policyId": "6960be11-e3a6-46cc-bf66-933c57c2af5d",
                    "complianceId": "2ab6eea3-660b-48c4-a836-53347e702faf",
                    "sectionLabel": "Data protection",
                    "sectionViewOrder": 5,
                    "requirementViewOrder": 2,
                    "systemDefault": True,
                    "customAssigned": False
                }
            ],
            "labels": [
                "CODE",
                "KARTIK"
            ],
            "enabled": True,
            "createdOn": 1634832351154,
            "createdBy": "Prisma Cloud System Admin",
            "lastModifiedOn": 1654106204620,
            "lastModifiedBy": "User",
            "ruleLastModifiedOn": 1639652340813,
            "deleted": False,
            "owner": "Prisma Cloud",
            "policyMode": "redlock_default",
            "policyCategory": "risk",
            "policyClass": "exposure",
            "remediable": False
        },
        {
            "policyId": "3dc2478c-bf25-4383-aaa1-30feb5cda586",
            "name": "Application gateway does not use WAF in Detection or Prevention modes",
            "policyType": "config",
            "policySubTypes": [
                "build"
            ],
            "systemDefault": True,
            "description": "Ensure that Application Gateway uses WAF in \"Detection\" or \"Prevention\" modes",
            "severity": "medium",
            "rule": {
                "parameters": {
                    "withIac": "true",
                    "savedSearch": "false"
                },
                "type": "Config",
                "children": [
                    {
                        "criteria": "{\"category\":\"Networking\",\"resourceTypes\":[\"azurerm_web_application_firewall_policy\"]}",
                        "type": "build",
                        "metadata": {
                            "checkovId": "CKV_AZURE_122"
                        },
                        "recommendation": "Refer the documentation for more details,\nhttps://docs.bridgecrew.io/docs/ensure-that-application-gateway-uses-waf-in-detection-or-prevention-modes"
                    }
                ]
            },
            "recommendation": "",
            "cloudType": "azure",
            "labels": [
                "pcsup"
            ],
            "enabled": True,
        },
        {
            "policyId": "c11ce08c-b93e-4e11-8d1c-e5a1339139d1",
            "name": "CUSTOM 3: Ensure subnet is associated with NSG",
            "policyType": "config",
            "policySubTypes": [
                "build"
            ],
            "systemDefault": False,
            "description": "Every subnet should be associated with NSG for controlling access to \nresources within the subnet.\n",
            "severity": "high",
            "rule": {
                "name": "CUSTOM 3: Ensure subnet is associated with NSG",
                "parameters": {
                    "withIac": "true",
                    "savedSearch": "false"
                },
                "type": "Config",
                "children": [
                    {
                        "type": "build",
                        "metadata": {
                            "code": "metadata:\n  name: 'CUSTOM 3: Ensure subnet is associated with NSG'\n  guidelines: \"Every subnet should be associated with NSG for controlling access to\\\n    \\ \\nresources within the subnet.\\n\"\n  category: networking\n  severity: high\nscope:\n  provider: azure\ndefinition:\n  and:\n  - cond_type: connection\n    resource_types:\n    - azurerm_subnet_network_security_group_association\n    connected_resource_types:\n    - azurerm_subnet\n    - azurerm_network_security_group\n    operator: exists\n  - cond_type: filter\n    attribute: resource_type\n    value:\n    - azurerm_subnet\n    operator: within\n"
                        },
                        "recommendation": ""
                    }
                ]
            },
            "recommendation": "",
            "cloudType": "azure",
            "labels": [
                "CODE",
                "KARTIK"
            ],
            "enabled": True,
            "createdOn": 1653999690681,
            "createdBy": "User",
            "lastModifiedOn": 1654065712108,
            "lastModifiedBy": "User",
            "ruleLastModifiedOn": 1653999690681,
            "deleted": False,
            "owner": "Tenant Name",
            "policyMode": "custom",
            "policyCategory": "risk",
            "policyClass": "exposure",
            "remediable": False
        },
        {
            "policyId": "0e4c576e-c934-4af3-8592-a53920e71ffb",
            "name": "CUSTOM POLICY2: Ensure subnet is associated with NSG",
            "policyType": "config",
            "policySubTypes": [
                "build"
            ],
            "systemDefault": False,
            "description": "Every subnet should be associated with NSG for controlling access to \nresources within the subnet.\n",
            "severity": "high",
            "rule": {
                "name": "CUSTOM POLICY2: Ensure subnet is associated with NSG",
                "parameters": {
                    "withIac": "true",
                    "savedSearch": "false"
                },
                "type": "Config",
                "children": [
                    {
                        "type": "build",
                        "metadata": {
                            "code": "metadata:\n  name: 'CUSTOM POLICY2: Ensure subnet is associated with NSG'\n  guidelines: \"Every subnet should be associated with NSG for controlling access to\\\n    \\ \\nresources within the subnet.\\n\"\n  category: networking\n  severity: high\nscope:\n  provider: azure\ndefinition:\n  and:\n  - cond_type: connection\n    resource_types:\n    - azurerm_subnet_network_security_group_association\n    connected_resource_types:\n    - azurerm_subnet\n    - azurerm_network_security_group\n    operator: exists\n  - cond_type: filter\n    attribute: resource_type\n    value:\n    - azurerm_subnet\n    operator: within\n"
                        },
                        "recommendation": ""
                    }
                ]
            },
            "recommendation": "",
            "cloudType": "azure",
            "labels": [
                "CODE",
                "KARTIK"
            ],
            "enabled": True,
            "createdOn": 1653003961960,
            "createdBy": "User",
            "lastModifiedOn": 1653003961960,
            "lastModifiedBy": "User",
            "ruleLastModifiedOn": 1653003961960,
            "deleted": False,
            "owner": "Tenant Name",
            "policyMode": "custom",
            "policyCategory": "risk",
            "policyClass": "exposure",
            "remediable": False
        },
    ]


if __name__ == '__main__':
    unittest.main()



import os
import unittest

from checkov.runner_filter import RunnerFilter
from checkov.terraform.runner import Runner
from checkov.terraform.checks.data.aws.IAMManagedAdminPolicy import check


class TestIAMManagedAdminPolicy(unittest.TestCase):

    def test(self):
        runner = Runner()
        current_dir = os.path.dirname(os.path.realpath(__file__))

        test_files_dir = os.path.join(current_dir, "example_IAMManagedAdminPolicy")
        report = runner.run(root_folder=test_files_dir,
                            runner_filter=RunnerFilter(checks=[check.id]))
        summary = report.get_summary()

        passing_resources = {
            "aws_iam_policy.pass1",
            "aws_iam_policy.pass2",
        }

        failing_resources = {
            "aws_iam_policy.fail1",
            "aws_iam_policy.fail2",
        }

        passed_check_resources = {c.resource for c in report.passed_checks}
        failed_check_resources = {c.resource for c in report.failed_checks}

        self.assertEqual(summary["passed"], 2)
        self.assertEqual(summary["failed"], 2)
        self.assertEqual(summary["skipped"], 0)
        self.assertEqual(summary["parsing_errors"], 0)

        self.assertEqual(passing_resources, passed_check_resources)
        self.assertEqual(failing_resources, failed_check_resources)


if __name__ == '__main__':
    unittest.main()


import unittest

from checkov.terraform.checks.resource.aws.LambdaInVPC import check
from checkov.common.models.enums import CheckResult

import hcl2


class TestLambdaInVPC(unittest.TestCase):

    def test_failure(self):
        hcl_res = hcl2.loads("""
            resource "aws_lambda_function" "test_lambda" {
              filename      = "lambda_function_payload.zip"
              function_name = "lambda_function_name"
              role          = aws_iam_role.iam_for_lambda.arn
              handler       = "exports.test"

              source_code_hash = filebase64sha256("lambda_function_payload.zip")

              runtime = "nodejs12.x"

              environment {
                variables = {
                  foo = "bar"
                }
              }
            }
        """)
        resource_conf = hcl_res['resource'][0]['aws_lambda_function']['test_lambda']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_success(self):
        hcl_res = hcl2.loads("""
        resource "aws_lambda_function" "test_lambda" {
              filename      = "lambda_function_payload.zip"
              function_name = "lambda_function_name"
              role          = aws_iam_role.iam_for_lambda.arn
              handler       = "exports.test"
              source_code_hash = filebase64sha256("lambda_function_payload.zip")
              runtime = "nodejs12.x"
              vpc_config {
                # Every subnet should be able to reach an EFS mount target in the same Availability Zone. Cross-AZ mounts are not permitted.
                subnet_ids         = [aws_subnet.subnet_for_lambda.id]
                security_group_ids = [aws_security_group.sg_for_lambda.id]
              }
              environment {
                variables = {
                  foo = "bar"
                }
              }
            }
        """)
        resource_conf = hcl_res['resource'][0]['aws_lambda_function']['test_lambda']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)


if __name__ == '__main__':
    unittest.main()
